@@ -15,14 +15,22 @@
  * The GNU General Public License can be found at
  * http://www.gnu.org/licenses/gpl.html.
  */
+#define _GNU_SOURCE
+
 #include <stdio.h>
 #include <string.h>
+
 #ifndef __OpenBSD__
-#include <pty.h>
+# ifndef USE_UNIX98_PTY
+#  include <pty.h>
+# endif /* !USE_UNIX98_PTY */
 #else
-#include <termios.h>
-#include <util.h>
+# include <util.h>
 #endif
+
+
+#include <termios.h>
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <dirent.h>
@@ -46,6 +54,10 @@
 #include <stdint.h>
 #include <tiffio.h>
 
+#ifndef O_LARGEFILE
+#define O_LARGEFILE 0
+#endif
+
 #include <spandsp.h>
 #ifdef STATICLIBS
 #include <iax-client.h>
@@ -101,11 +113,11 @@ static char dspnowaudiofile[50];
 static char iaxnowaudiofile[50];
 static struct timeval now, lasthangup, nextaudio;	/* target time for next audio */
 static int window = 1000;		/* u-sec prior to target that we will let audio go */
-static int refresh = 60;		/* negotiated refresh */
 static int codec = AST_FORMAT_SLINEAR;	/* negotiated codec */
 
 static int port = 4569;
 static int refreshreq = 300;		/* requested refresh */
+static int refresh = 300;		/* negotiated refresh, initialize equal to refreshreq */
 static int codecreq = AST_FORMAT_SLINEAR; /* requested codec */
 static char devlink[64];
 static char devowner[64];		/* owner:group for the slave tty */
@@ -119,6 +131,7 @@ static char call_time[16];
 static char call_date[16];
 static int record = 0;
 static int replay = 0;
+static int nojitterbuffer = 0;
 
 struct modem {
     int pid;
@@ -154,6 +167,18 @@ printlog(FILE *fp, char *fmt, ...)
   va_end(ap);
 }
 
+void
+checklog(char* filename)
+{
+    struct stat fs;
+    if (stat(filename, &fs) == 0) {
+	if (fs.st_size > 0x40000000) {
+	    char oname[256];
+	    snprintf(oname, sizeof(oname), "%s.old", filename);
+	    rename(filename, oname);
+	}
+    }
+}
 
 void
 cleanup(int sig)
@@ -325,6 +350,10 @@ setconfigline(char *line)
 	replay = 1;
 	printlog(LOG_INFO, "Enabling replay\n");
     }
+    if (strncasecmp(line, "nojitterbuffer", 12) == 0) {
+	nojitterbuffer = 1;
+	printlog(LOG_INFO, "Disabling jitterbuffer\n");
+    }
 }
 
 void
@@ -332,7 +361,7 @@ readconfig(const char *name)
 {
     if (!strchr(name, '%')) {
 	char filename[256];
-	snprintf(filename, 255, "/etc/iaxmodem/%s", name);
+	snprintf(filename, sizeof(filename), "/etc/iaxmodem/%s", name);
 	int fd = open(filename, O_RDONLY);
 	if (fd < 0) {
 	    printlog(LOG_ERROR, "Cannot open %s.\n", filename);
@@ -342,7 +371,7 @@ readconfig(const char *name)
 	char line[1024];
 	memset(line, 0, 1024);
 	while ((cc = read(fd, &line[pos], 1))) {
-	    if (line[pos] == '\n' || pos == 1024) {
+	    if (line[pos] == '\n' || pos == 1023) {
 		line[pos] = '\0';
 		setconfigline(line);
 		memset(line, 0, 1024);
@@ -352,7 +381,10 @@ readconfig(const char *name)
 	if (strlen(line)) {
 	    setconfigline(line);
 	}
-	if (replay && record) record = 0;
+	if (replay && record) {
+	    record = 0;
+	    printlog(LOG_INFO, "Disabling record\n");
+	}
 	close(fd);
     }
 }
@@ -373,7 +405,9 @@ at_tx_handler(at_state_t *s, void *user_data, const uint8_t *buf, size_t len)
 	  buffer.
 	*/
 	if (tcflush(amaster, TCOFLUSH)) {
-	    printlog(LOG_ERROR, "Unable to flush pty buffer: %s\n", strerror(errno));
+	    printlog(LOG_ERROR, "Unable to flush pty master buffer: %s\n", strerror(errno));
+	} else if (tcflush(aslave, TCOFLUSH)) {
+	    printlog(LOG_ERROR, "Unable to flush pty slave buffer: %s\n", strerror(errno));
 	} else {
 	    printlog(LOG_INFO, "Successfully flushed pty buffer\n");
 	}
@@ -392,7 +426,7 @@ t31_call_control_handler(t31_state_t *s, void *user_data, int op, const char *nu
 		printlog(LOG_INFO, "Dialing '%s'\n", num);
 		session[0] = iax_session_new();
 		char ich[256];
-		snprintf(ich, 255, "%s:%s@%s/%s", regpeer, regsecret, server, num);
+		snprintf(ich, sizeof(ich), "%s:%s@%s/%s", regpeer, regsecret, server, num);
 		iax_call(session[0], cidnumber, cidname, ich, NULL, 0, codecreq, CODEC_SUPPORT);
 		modemstate = MODEM_CALLING;
 	    }
@@ -569,8 +603,9 @@ iaxmodem(const char *config, int nolog)
 	close(fd);
 
 	/* Redirect stderr to the log file */
-	snprintf(logfile, 255, "/var/log/iaxmodem/%s", config);
-	fd = open(logfile, O_WRONLY | O_APPEND | O_CREAT, logmode);
+	snprintf(logfile, sizeof(logfile), "/var/log/iaxmodem/%s", config);
+	checklog(logfile);
+	fd = open(logfile, O_WRONLY | O_APPEND | O_CREAT | O_LARGEFILE, logmode);
 	if (fd < 0) {
 	    printlog(LOG_ERROR, "Error: could not open %s: %s\n", logfile, strerror(errno));
 	} else {
@@ -593,7 +628,7 @@ iaxmodem(const char *config, int nolog)
     gettimeofday(&lastdtedata, NULL);
     lastdtedata.tv_sec -= 60;
     gettimeofday(&lastreg, NULL);
-    lastreg.tv_sec -= 60;
+    lastreg.tv_sec -= refreshreq;
     gettimeofday(&lastring, NULL);
     lastring.tv_sec -= 60;
     gettimeofday(&lasthangup, NULL);
@@ -613,10 +648,10 @@ iaxmodem(const char *config, int nolog)
     int i;
     unsigned int last_ts = 0;
 
-    snprintf(dspnowaudiofile, 49, "/tmp/%s-dsp.raw.recording", regpeer);
-    snprintf(iaxnowaudiofile, 49, "/tmp/%s-iax.raw.recording", regpeer);
-    snprintf(dspaudiofile, 49, "/tmp/%s-dsp.raw", regpeer);
-    snprintf(iaxaudiofile, 49, "/tmp/%s-iax.raw", regpeer);
+    snprintf(dspnowaudiofile, sizeof(dspnowaudiofile), "/tmp/%s-dsp.raw.recording", regpeer);
+    snprintf(iaxnowaudiofile, sizeof(iaxnowaudiofile), "/tmp/%s-iax.raw.recording", regpeer);
+    snprintf(dspaudiofile, sizeof(dspaudiofile), "/tmp/%s-dsp.raw", regpeer);
+    snprintf(iaxaudiofile, sizeof(iaxaudiofile), "/tmp/%s-iax.raw", regpeer);
 
     /* Get the uid/gid corresponding to devowner */
     devgroup = strchr(devowner, ':');
@@ -665,16 +700,60 @@ iaxmodem(const char *config, int nolog)
 	}
     }
 
+#ifndef USE_UNIX98_PTY
     if (openpty(&amaster, &aslave, NULL, NULL, NULL)) {
         printlog(LOG_ERROR, "Fatal error: failed to initialize pty\n");
         cleanup(-1);
     }
+
     char *stty = ttyname(aslave);
+    if (stty == NULL) {
+      printlog(LOG_ERROR, "Fatal error: failed to obtain slave pty filename\n");
+      cleanup(-1);
+    }
+    stty = strdup(stty);
+
+#else /* USE_UNIX98_PTY */
+
+    amaster = posix_openpt(O_RDWR | O_NOCTTY);
+    if (amaster < 0) {
+      printlog(LOG_ERROR, "Fatal error: failed to initialize UNIX98 master pty\n");
+      cleanup(-1);
+    }
+
+    if (grantpt(amaster) < 0) {
+      printlog(LOG_ERROR, "Fatal error: failed to grant access to slave pty\n");
+      cleanup(-1);
+    }
+
+    if (unlockpt(amaster) < 0) {
+      printlog(LOG_ERROR, "Fatal error: failed to unlock slave pty\n");
+      cleanup(-1);
+    }
+
+    char *stty = ptsname(amaster);
+    if (stty == NULL) {
+      printlog(LOG_ERROR, "Fatal error: failed to obtain slave pty filename\n");
+      cleanup(-1);
+    }
+    stty = strdup(stty);
+
+    aslave = open(stty, O_RDWR);
+    if (aslave < 0) {
+      printlog(LOG_ERROR, "Fatal error: failed to open slave pty %s\n", stty);
+      cleanup(-1);
+    }
+
+#endif /* !USE_UNIX98_PTY */
+
     printlog(LOG_INFO, "Opened pty, slave device: %s\n", stty);
     if (!unlink(devlink)) {
 	printlog(LOG_ERROR, "Removed old %s\n", devlink);
     }
-    if (symlink(stty, devlink)) {
+
+    int ret = symlink(stty, devlink);
+    free(stty);
+    if (ret < 0) {
 	printlog(LOG_ERROR, "Fatal error: failed to create %s symbolic link\n", devlink);
 	cleanup(-1);
     }
@@ -729,6 +808,9 @@ iaxmodem(const char *config, int nolog)
      * Again, debugging should be minimal for normal use as the writes slow things too much.
      */
 //    t31_state.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
+//    t31_state.v17rx.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
+//    t31_state.v29rx.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
+//    t31_state.v27ter_rx.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
 
     int selectfd, selectretval, selectblock, avail, audiobalance = 0, skew = 0;
     struct timeval tv;
@@ -803,7 +885,7 @@ iaxmodem(const char *config, int nolog)
 	 * presumes, then, that in those cases the rest of the loop has served as
 	 * the throttle for that iteration.
 	 */
-	if (tv.tv_sec > 0 || tv.tv_usec > 0) {
+	if ((tv.tv_sec == 0 && tv.tv_usec > 0) || (tv.tv_sec > 0 && tv.tv_sec < 3600)) {
 	    int nexteventms = iax_time_to_next_event();
 	    /*
 	     * iax_time_to_next_event will return 0 when a response is "pending",
@@ -930,9 +1012,18 @@ iaxmodem(const char *config, int nolog)
 	/*
 	 * Did the DCE timeout in sending CONNECT response?
 	 */
-	if (modemstate == MODEM_CALLING && now.tv_sec > lastdtedata.tv_sec + 45) {
+	if (modemstate == MODEM_CALLING && timediff(now, lastdtedata) >= 45000000) {
 	    t31_call_event(&t31_state, AT_CALL_EVENT_NO_ANSWER);
-	    iax_hangup(session[0], "Give up");
+	    /*
+	     * One would think that this would be appropriate here:
+	     *
+	     * iax_hangup(session[0], "Give up");
+	     *
+	     * Yet, it is sometimes not.  It's probably a bug in libiax2, and is 
+	     * triggered by an incoming call occurring simultaneous with an outgoing 
+	     * one.  In any case, omitting it seems harmless, as iax_destroy seems
+	     * to do the necessary job.
+	     */
 	    iax_destroy(session[0]);
 	    phonestate = PHONE_FREED;
 	    modemstate = MODEM_ONHOOK;
@@ -946,7 +1037,7 @@ iaxmodem(const char *config, int nolog)
 		switch (iaxevent->etype) {
 		    case IAX_EVENT_REGACK:
 			printlog(LOG_INFO, "Registration completed successfully.\n");
-			if (iaxevent->ies.refresh) refresh = iaxevent->ies.refresh;
+			if (iaxevent->ies.refresh > 0) refresh = iaxevent->ies.refresh;
 			regstate = REGISTERED;
 			break;
 		    case IAX_EVENT_REGREJ:
@@ -1048,11 +1139,16 @@ iaxmodem(const char *config, int nolog)
 			printlog(LOG_INFO, "Incoming call connected %s, %s, %s.\n", iaxevent->ies.called_number, iaxevent->ies.calling_number, iaxevent->ies.calling_name);
 			strftime(call_date, sizeof(call_date), "%m%d", localtime((time_t*) &now.tv_sec));
 			strftime(call_time, sizeof(call_time), "%H%M", localtime((time_t*) &now.tv_sec));
-			at_set_call_info(&t31_state.at_state, call_date, call_time, 
-				iaxevent->ies.calling_name,
-				iaxevent->ies.calling_number,
-				iaxevent->ies.calling_ani,
-				iaxevent->ies.called_number);
+			at_reset_call_info(&t31_state.at_state);
+			at_set_call_info(&t31_state.at_state, "DATE", call_date);
+			at_set_call_info(&t31_state.at_state, "TIME", call_time);
+			at_set_call_info(&t31_state.at_state, "NAME", iaxevent->ies.calling_name);
+			at_set_call_info(&t31_state.at_state, "NMBR", iaxevent->ies.calling_number);
+			at_set_call_info(&t31_state.at_state, "ANID", iaxevent->ies.calling_ani);
+			at_set_call_info(&t31_state.at_state, "USER", iaxevent->ies.username);
+			at_set_call_info(&t31_state.at_state, "PASS", iaxevent->ies.password);
+			at_set_call_info(&t31_state.at_state, "CDID", iaxevent->ies.called_context);
+			at_set_call_info(&t31_state.at_state, "NDID", iaxevent->ies.called_number);
 			t31_call_event(&t31_state, AT_CALL_EVENT_ALERTING);
 			break;
 		    case IAX_EVENT_BUSY:
@@ -1100,15 +1196,16 @@ iaxmodem(const char *config, int nolog)
 			 * Watch for IAX2 jitter... the 32-bit timestamp shouldn't ever "wrap" 
 			 * unless we expect a single call to last more than a month.
 			 */
-			if (last_ts && iaxevent->ts <= last_ts) {
+			if (!nojitterbuffer && last_ts && iaxevent->ts <= last_ts) {
 			    /*
 			     * We've already sent audio corresponding to this time.
 			     */
 			    break;
 			}
-			if (last_ts && iaxevent->ts != last_ts + 20) {
+			if (!nojitterbuffer && last_ts && iaxevent->ts != last_ts + 20) {
 			    printlog(LOG_ERROR, "IAX2 jitter - last_ts: %d, ts: %d\n", last_ts, iaxevent->ts);
 			    int16_t fillbuf[VOIP_PACKET_SIZE];
+			    memcpy(fillbuf, iaxbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
 			    while (last_ts + 20 < iaxevent->ts) {
 				/*
 				 * This audio packet skips ahead.  Oops, audio was lost.
@@ -1196,7 +1293,7 @@ iaxmodem(const char *config, int nolog)
 			if (t31_state.at_state.at_rx_mode == AT_MODE_OFFHOOK_COMMAND ||
 			    t31_state.at_state.at_rx_mode == AT_MODE_ONHOOK_COMMAND) {
 			    char dtmf[16];
-			    snprintf(dtmf, 15, "\r\nDTMF=%d\r\n", iaxevent->subclass - 48);
+			    snprintf(dtmf, sizeof(dtmf), "\r\nDTMF=%d\r\n", iaxevent->subclass - 48);
 			    at_tx_handler(&t31_state.at_state, NULL, (u_char*) dtmf, strlen(dtmf));
 			}
 			printlog(LOG_INFO, "Received DTMF '%d'\n", iaxevent->subclass - 48);
@@ -1319,12 +1416,12 @@ wait_for_modems()
 	pid = waitpid(-1, &status, 0);
 
 	if (pid > 0) {
-	    status >>= 8;
+	    if ((status & 0xFF) == 0x00) status >>= 8;
 	    if (status == SIGHUP) {
-		printlog(LOG_ERROR, "iaxmodem process %d ended, status %d, attempting to restart\n", pid, status);
+		printlog(LOG_ERROR, "iaxmodem process %d ended w/SIGHUP, attempting to restart\n", pid);
 		restart_modem(pid);
 	    } else {
-		printlog(LOG_ERROR, "iaxmodem process %d ended, status %d\n", pid, status);
+		printlog(LOG_ERROR, "iaxmodem process %d ended, status 0x%X\n", pid, status);
 		remove_modem(pid);
 	    }
 	} else {
@@ -1349,9 +1446,11 @@ spawn_modems(void)
 {
     DIR *cfdir;
     struct dirent *cf;
+    struct stat st;
     int len;
     int pid = -1;
     char *config = NULL;
+    char filename[256];
 
     /* List configuration files */
     cfdir = opendir("/etc/iaxmodem");
@@ -1368,6 +1467,20 @@ spawn_modems(void)
 	if ((cf->d_name[0] == '.') || (cf->d_name[0] == '#') || (cf->d_name[len - 1] == '~'))
 	  continue;
 
+	/* Skip anything that isn't a file */
+	snprintf(filename, sizeof(filename), "/etc/iaxmodem/%s", cf->d_name);
+	if (stat(filename, &st) < 0)
+	  {
+	    printlog(LOG_INFO, "Could not stat configuration file %s: %s\n", cf->d_name, strerror(errno));
+	    continue;
+	  }
+
+	if (!S_ISREG(st.st_mode))
+	  {
+	    printlog(LOG_INFO, "bouh !\n");
+	    continue;
+	  }
+
 	config = strdup(cf->d_name);
 
 	/* Spawn modem processes */
@@ -1427,7 +1540,8 @@ ctrl_hup_sighandler(int sig)
     printlog(LOG_ERROR, "Configuration changed, restarting...\n");
 
     /* Reopen the log file */
-    fd = open("/var/log/iaxmodem/iaxmodem", O_WRONLY | O_APPEND | O_CREAT, logmode);
+    checklog("/var/log/iaxmodem/iaxmodem");
+    fd = open("/var/log/iaxmodem/iaxmodem", O_WRONLY | O_APPEND | O_CREAT | O_LARGEFILE, logmode);
     if (fd < 0) {
       printlog(LOG_ERROR, "Error: could not open /var/log/iaxmodem/iaxmodem: %s\n", strerror(errno));
     } else {
@@ -1458,7 +1572,7 @@ main(int argc, char** argv)
 
     /* If a config is specified, behave in non-daemon mode */
     if (argc == 2) {
-      snprintf(config, 255, "%s", argv[1]);
+      snprintf(config, sizeof(config), "%s", argv[1]);
       iaxmodem(config, 1);
     }
 
@@ -1504,7 +1618,8 @@ main(int argc, char** argv)
     close(fd);
 
     /* Redirect stderr to the log file */
-    fd = open("/var/log/iaxmodem/iaxmodem", O_WRONLY | O_APPEND | O_CREAT, logmode);
+    checklog("/var/log/iaxmodem/iaxmodem");
+    fd = open("/var/log/iaxmodem/iaxmodem", O_WRONLY | O_APPEND | O_CREAT | O_LARGEFILE, logmode);
     if (fd < 0) {
       printlog(LOG_ERROR, "Error: could not open /var/log/iaxmodem/iaxmodem: %s\n", strerror(errno));
     } else {
