@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: adsi.c,v 1.35 2006/10/24 16:38:32 steveu Exp $
+ * $Id: adsi.c,v 1.39 2007/03/26 12:57:41 steveu Exp $
  */
 
 /*! \file */
@@ -36,7 +36,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 #include <assert.h>
 
 #include "spandsp/telephony.h"
@@ -48,6 +53,7 @@
 #include "spandsp/fsk.h"
 #include "spandsp/tone_detect.h"
 #include "spandsp/tone_generate.h"
+#include "spandsp/super_tone_rx.h"
 #include "spandsp/dtmf.h"
 #include "spandsp/adsi.h"
 
@@ -119,7 +125,7 @@ static int adsi_tx_get_bit(void *user_data)
     }
     else
     {
-        bit = 2;
+        bit = PUTBIT_END_OF_DATA;
         if (s->tx_signal_on)
         {
             /* The FSK should now be switched off. */
@@ -420,18 +426,26 @@ void adsi_rx_init(adsi_rx_state_t *s,
 int adsi_tx(adsi_tx_state_t *s, int16_t *amp, int max_len)
 {
     int len;
+    int lenx;
 
     len = tone_gen(&(s->alert_tone_gen), amp, max_len);
-    switch (s->standard)
+    if (s->tx_signal_on)
     {
-    case ADSI_STANDARD_CLIP_DTMF:
-        if (len < max_len)
-            len += dtmf_tx(&(s->dtmftx), amp, max_len - len);
-        break;
-    default:
-        if (len < max_len  &&  s->tx_signal_on)
-            len += fsk_tx(&(s->fsktx), amp + len, max_len - len);
-        break;
+        switch (s->standard)
+        {
+        case ADSI_STANDARD_CLIP_DTMF:
+            if (len < max_len)
+                len += dtmf_tx(&(s->dtmftx), amp, max_len - len);
+            break;
+        default:
+            if (len < max_len)
+            {
+                if ((lenx = fsk_tx(&(s->fsktx), amp + len, max_len - len)) <= 0)
+                    s->tx_signal_on = FALSE;
+                len += lenx;
+            }
+            break;
+        }
     }
     return len;
 }
