@@ -24,7 +24,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: fax.c,v 1.51 2006/10/24 13:45:25 steveu Exp $
+ * $Id: fax.c,v 1.53 2007/04/08 08:16:17 steveu Exp $
  */
 
 /*! \file */
@@ -37,7 +37,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 #include <assert.h>
 #include <fcntl.h>
 #include <time.h>
@@ -527,8 +532,14 @@ void fax_set_tep_mode(fax_state_t *s, int use_tep)
 }
 /*- End of function --------------------------------------------------------*/
 
-int fax_init(fax_state_t *s, int calling_party)
+fax_state_t *fax_init(fax_state_t *s, int calling_party)
 {
+    if (s == NULL)
+    {
+        if ((s = (fax_state_t *) malloc(sizeof(*s))) == NULL)
+            return NULL;
+    }
+
     memset(s, 0, sizeof(*s));
     span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
     span_log_set_protocol(&s->logging, "FAX");
@@ -593,7 +604,7 @@ int fax_init(fax_state_t *s, int calling_party)
         s->fax_audio_tx_log = open(buf, O_CREAT | O_TRUNC | O_WRONLY, 0666);
     }
 #endif
-    return 0;
+    return s;
 }
 /*- End of function --------------------------------------------------------*/
 
