@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: hdlc.c,v 1.42 2006/10/24 13:45:25 steveu Exp $
+ * $Id: hdlc.c,v 1.45 2007/03/29 12:28:35 steveu Exp $
  */
 
 /*! \file */
@@ -285,20 +285,16 @@ static void rx_flag_or_abort(hdlc_rx_state_t *s)
                     {
                         s->rx_frames++;
                         s->rx_bytes += s->len - s->crc_bytes;
-                        s->frame_handler(s->user_data,
-                                         TRUE,
-                                         s->buffer,
-                                         s->len - s->crc_bytes);
+                        s->len -= s->crc_bytes;
+                        s->frame_handler(s->user_data, TRUE, s->buffer, s->len);
                     }
                     else
                     {
                         s->rx_crc_errors++;
                         if (s->report_bad_frames)
                         {
-                            s->frame_handler(s->user_data,
-                                             FALSE,
-                                             s->buffer,
-                                             s->len - s->crc_bytes);
+                            s->len -= s->crc_bytes;
+                            s->frame_handler(s->user_data, FALSE, s->buffer, s->len);
                         }
                     }
                     s->framing_ok_announced = FALSE;
@@ -308,10 +304,12 @@ static void rx_flag_or_abort(hdlc_rx_state_t *s)
                     /* Frame too short or too long */
                     if (s->report_bad_frames)
                     {
-                        s->frame_handler(s->user_data,
-                                         FALSE,
-                                         s->buffer,
-                                         s->len - s->crc_bytes);
+                        s->len -= s->crc_bytes;
+                        /* Don't let the length go negative, or it will be confused
+                           with one of the special conditions. */
+                        if (s->len < 0)
+                            s->len = 0;
+                        s->frame_handler(s->user_data, FALSE, s->buffer, s->len);
                     }
                     s->rx_length_errors++;
                 }
@@ -358,10 +356,11 @@ void hdlc_rx_put_bit(hdlc_rx_state_t *s, int new_bit)
     }
     else
     {
+        s->num_bits++;
         if (s->flags_seen >= s->framing_ok_threshold)
         {
             s->byte_in_progress = (s->byte_in_progress | (s->raw_bit_stream & 0x100)) >> 1;
-            if (++s->num_bits == 8)
+            if (s->num_bits == 8)
             {
                 /* Ensure we do not overflow our buffer */
                 if (s->len < (int) sizeof(s->buffer))
@@ -371,10 +370,6 @@ void hdlc_rx_put_bit(hdlc_rx_state_t *s, int new_bit)
                 s->num_bits = 0;
             }
         }
-        else
-        {
-            s->num_bits++;
-        }
     }
 }
 /*- End of function --------------------------------------------------------*/
@@ -459,7 +454,7 @@ int hdlc_tx_frame(hdlc_tx_state_t *s, const uint8_t *frame, int len)
     }
     memcpy(s->buffer + s->len, frame, len);
     if (s->crc_bytes == 2)
-        s->crc = crc_itu16_calc(frame, len, s->crc);
+        s->crc = crc_itu16_calc(frame, len, (uint16_t) s->crc);
     else
         s->crc = crc_itu32_calc(frame, len, s->crc);
     if (s->progressive)
@@ -573,7 +568,10 @@ int hdlc_tx_get_byte(hdlc_tx_state_t *s)
             }
         }
         byte_in_progress = s->buffer[s->pos++];
-        for (i = 0;  i < 8;  i++)
+        i = bottom_bit(byte_in_progress | 0x100);
+        s->octets_in_progress <<= i;
+        byte_in_progress >>= i;
+        for (  ;  i < 8;  i++)
         {
             s->octets_in_progress = (s->octets_in_progress << 1) | (byte_in_progress & 0x01);
             byte_in_progress >>= 1;
