@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: power_meter.c,v 1.17 2006/10/24 13:45:26 steveu Exp $
+ * $Id: power_meter.c,v 1.19 2007/04/10 16:12:19 steveu Exp $
  */
 
 /*! \file */
@@ -37,7 +37,12 @@
 #include <fcntl.h>
 #include <string.h>
 #include <float.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <assert.h>
 
 #include "spandsp/telephony.h"
@@ -49,6 +54,8 @@ power_meter_t *power_meter_init(power_meter_t *s, int shift)
         return NULL;
     s->shift = shift;
     s->reading = 0;
+    s->high_sample = 0;
+    s->low_samples = 0;
     return s;
 }
 /*- End of function --------------------------------------------------------*/
@@ -63,6 +70,20 @@ power_meter_t *power_meter_damping(power_meter_t *s, int shift)
 int32_t power_meter_update(power_meter_t *s, int16_t amp)
 {
     s->reading += ((amp*amp - s->reading) >> s->shift);
+    if (abs(amp) < abs(s->high_sample / 20))
+    {
+        if (++s->low_samples > 160)
+        {
+            s->reading = 0;
+            s->high_sample = 0;
+            s->low_samples = 0;
+        }
+    }
+    else
+    {
+	s->low_samples = 0;
+	if (abs(s->high_sample) < abs(amp)) s->high_sample = amp;
+    }
     return s->reading;
 }
 /*- End of function --------------------------------------------------------*/
@@ -90,7 +111,13 @@ int32_t power_meter_level_dbov(float level)
 }
 /*- End of function --------------------------------------------------------*/
 
-float power_meter_dbm0(power_meter_t *s)
+int32_t power_meter_current(power_meter_t *s)
+{
+    return s->reading;
+}
+/*- End of function --------------------------------------------------------*/
+
+float power_meter_current_dbm0(power_meter_t *s)
 {
     if (s->reading <= 0)
         return FLT_MIN;
@@ -99,7 +126,7 @@ float power_meter_dbm0(power_meter_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-float power_meter_dbov(power_meter_t *s)
+float power_meter_current_dbov(power_meter_t *s)
 {
     if (s->reading <= 0)
         return FLT_MIN;
