@@ -22,18 +22,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: dds.h,v 1.9 2006/10/24 13:45:28 steveu Exp $
+ * $Id: dds.h,v 1.12 2007/04/08 08:16:17 steveu Exp $
  */
 
 /*! \file */
 
-#if !defined(_DDS_H_)
-#define _DDS_H_
+#if !defined(_SPANDSP_DDS_H_)
+#define _SPANDSP_DDS_H_
 
 #include "complex.h"
 
-#ifdef __cplusplus
-extern "C" {
+#if defined(__cplusplus)
+extern "C"
+{
 #endif
 
 /*! \brief Find the phase rate value to achieve a particular frequency.
@@ -137,7 +138,7 @@ complexf_t dds_complexf(uint32_t *phase_acc, int32_t phase_rate);
 */
 complexf_t dds_complex_modf(uint32_t *phase_acc, int32_t phase_rate, float scale, int32_t phase);
 
-#ifdef __cplusplus
+#if defined(__cplusplus)
 }
 #endif
 
