@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,18 +22,29 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t30.h,v 1.40 2006/05/13 01:57:43 steveu Exp $
+ * $Id: t30.h,v 1.72 2007/05/01 14:34:51 steveu Exp $
  */
 
 /*! \file */
 
-#if !defined(_T30_H_)
-#define _T30_H_
+#if !defined(_SPANDSP_T30_H_)
+#define _SPANDSP_T30_H_
 
 /*! \page t30_page T.30 FAX protocol handling
 
 \section t30_page_sec_1 What does it do?
-The T.30 protocol is the core protocol used for FAX transmission.
+The T.30 protocol is the core protocol used for FAX transmission. This module
+implements most of its key featrues. It does not interface to the outside work.
+Seperate modules do that for T.38, analogue line, and other forms of FAX
+communication.
+
+Current features of this module include:
+
+    - FAXing to and from multi-page TIFF/F files, whose images are one of the standard
+      FAX sizes.
+    - T.4 1D (MH), T.4 2D,(MR) and T.6 (MMR) compression.
+    - Error correction (ECM)
+    - All standard resolutions and page sizes
 
 \section t30_page_sec_2 How does it work?
 
@@ -117,7 +127,9 @@ There are many other commonly encountered variations between machines, including
     * DCN usage
 */
 
-#define MAXFRAME            252
+#define T30_MAX_DIS_DTC_DCS_LEN     22
+#define T30_MAX_IDENT_LEN           21
+#define T30_MAX_LOCAL_NSF_LEN       100
 
 typedef struct t30_state_s t30_state_t;
 
@@ -163,8 +175,9 @@ typedef int (t30_document_handler_t)(t30_state_t *s, void *user_data, int status
     \param user_data An opaque pointer.
     \param type The modem, tone or silence to be sent or received.
     \param short_train TRUE if the short training sequence should be used (where one exists).
+    \param use_hdlc FALSE for bit stream, TRUE for HDLC framing.
 */
-typedef void (t30_set_handler_t)(void *user_data, int type, int short_train);
+typedef void (t30_set_handler_t)(void *user_data, int type, int short_train, int use_hdlc);
 
 /*!
     T.30 send HDLC handler.
@@ -180,76 +193,67 @@ typedef void (t30_send_hdlc_handler_t)(void *user_data, const uint8_t *msg, int
 */
 enum
 {
-    T30_ERR_OK = 0,         /* OK */
+    T30_ERR_OK = 0,         /*! OK */
 
-    /* External problems */
-    T30_ERR_CEDTONE,        /* The CED tone exceeded 5s */
-    T30_ERR_T0EXPIRED,      /* Timed out waiting for initial communication */
-    T30_ERR_T1EXPIRED,      /* Timed out waiting for the first message */
-    T30_ERR_T3EXPIRED,      /* Timed out waiting for procedural interrupt */
-    T30_ERR_HDLCCARR,       /* The HDLC carrier did not stop in a timely manner */
-    T30_ERR_CANNOTTRAIN,    /* Failed to train with any of the compatible modems */
-    T30_ERR_OPERINTFAIL,    /* Operator intervention failed */
-    T30_ERR_INCOMPATIBLE,   /* Far end is not compatible */
-    T30_ERR_NOTRXCAPABLE,   /* Far end is not receive capable */
-    T30_ERR_NOTTXCAPABLE,   /* Far end is not transmit capable */
-    T30_ERR_UNEXPECTED,     /* Unexpected message received */
-    T30_ERR_NORESSUPPORT,   /* Far end cannot receive at the resolution of the image */
-    T30_ERR_NOSIZESUPPORT,  /* Far end cannot receive at the size of image */
+    /* Link problems */
+    T30_ERR_CEDTONE,        /*! The CED tone exceeded 5s */
+    T30_ERR_T0_EXPIRED,     /*! Timed out waiting for initial communication */
+    T30_ERR_T1_EXPIRED,     /*! Timed out waiting for the first message */
+    T30_ERR_T3_EXPIRED,     /*! Timed out waiting for procedural interrupt */
+    T30_ERR_HDLC_CARRIER,   /*! The HDLC carrier did not stop in a timely manner */
+    T30_ERR_CANNOT_TRAIN,   /*! Failed to train with any of the compatible modems */
+    T30_ERR_OPERINTFAIL,    /*! Operator intervention failed */
+    T30_ERR_INCOMPATIBLE,   /*! Far end is not compatible */
+    T30_ERR_RX_INCAPABLE,   /*! Far end is not able to receive */
+    T30_ERR_TX_INCAPABLE,   /*! Far end is not able to transmit */
+    T30_ERR_NORESSUPPORT,   /*! Far end cannot receive at the resolution of the image */
+    T30_ERR_NOSIZESUPPORT,  /*! Far end cannot receive at the size of image */
+    T30_ERR_UNEXPECTED,     /*! Unexpected message received */
 
-    /* Internal problems */
-    T30_ERR_FILEERROR,      /* TIFF/F file cannot be opened */
-    T30_ERR_NOPAGE,         /* TIFF/F page not found */
-    T30_ERR_BADTIFF,        /* TIFF/F format is not compatible */
-    T30_ERR_UNSUPPORTED,    /* Unsupported feature */
+    /* TIFF file problems */
+    T30_ERR_FILEERROR,      /*! TIFF/F file cannot be opened */
+    T30_ERR_NOPAGE,         /*! TIFF/F page not found */
+    T30_ERR_BADTIFF,        /*! TIFF/F format is not compatible */
 
     /* Phase E status values returned to a transmitter */
-    T30_ERR_BADDCSTX,       /* Received bad response to DCS or training */
-    T30_ERR_BADPGTX,        /* Received a DCN from remote after sending a page */
-    T30_ERR_ECMPHDTX,       /* Invalid ECM response received from receiver */
-    T30_ERR_ECMRNRTX,       /* Timer T5 expired, receiver not ready */
-    T30_ERR_GOTDCNTX,       /* Received a DCN while waiting for a DIS */
-    T30_ERR_INVALRSPTX,     /* Invalid response after sending a page */
-    T30_ERR_NODISTX,        /* Received other than DIS while waiting for DIS */
-    T30_ERR_NXTCMDTX,       /* Timed out waiting for next send_page command from driver */
-    T30_ERR_PHBDEADTX,      /* Received no response to DCS, training or TCF */
-    T30_ERR_PHDDEADTX,      /* No response after sending a page */
+    T30_ERR_BADDCSTX,       /*! Received bad response to DCS or training */
+    T30_ERR_BADPGTX,        /*! Received a DCN from remote after sending a page */
+    T30_ERR_ECMPHDTX,       /*! Invalid ECM response received from receiver */
+    T30_ERR_T5_EXPIRED,     /*! Timed out waiting for receiver ready (ECM mode) */
+    T30_ERR_GOTDCNTX,       /*! Received a DCN while waiting for a DIS */
+    T30_ERR_INVALRSPTX,     /*! Invalid response after sending a page */
+    T30_ERR_NODISTX,        /*! Received other than DIS while waiting for DIS */
+    T30_ERR_PHBDEADTX,      /*! Received no response to DCS, training or TCF */
+    T30_ERR_PHDDEADTX,      /*! No response after sending a page */
 
     /* Phase E status values returned to a receiver */
-    T30_ERR_ECMPHDRX,       /* Invalid ECM response received from transmitter */
-    T30_ERR_GOTDCSRX,       /* DCS received while waiting for DTC */
-    T30_ERR_INVALCMDRX,     /* Unexpected command after page received */
-    T30_ERR_NOCARRIERRX,    /* Carrier lost during fax receive */
-    T30_ERR_NOEOLRX,        /* Timed out while waiting for EOL (end Of line) */
-    T30_ERR_NOFAXRX,        /* Timed out while waiting for first line */
-    T30_ERR_NXTCMDRX,       /* Timed out waiting for next receive page command */
-    T30_ERR_T2EXPDCNRX,     /* Timer T2 expired while waiting for DCN */
-    T30_ERR_T2EXPDRX,       /* Timer T2 expired while waiting for phase D */
-    T30_ERR_T2EXPFAXRX,     /* Timer T2 expired while waiting for fax page */
-    T30_ERR_T2EXPMPSRX,     /* Timer T2 expired while waiting for next fax page */
-    T30_ERR_T2EXPRRRX,      /* Timer T2 expired while waiting for RR command */
-    T30_ERR_T2EXPRX,        /* Timer T2 expired while waiting for NSS, DCS or MCF */
-    T30_ERR_DCNWHYRX,       /* Unexpected DCN while waiting for DCS or DIS */
-    T30_ERR_DCNDATARX,      /* Unexpected DCN while waiting for image data */
-    T30_ERR_DCNFAXRX,       /* Unexpected DCN while waiting for EOM, EOP or MPS */
-    T30_ERR_DCNPHDRX,       /* Unexpected DCN after EOM or MPS sequence */
-    T30_ERR_DCNRRDRX,       /* Unexpected DCN after RR/RNR sequence */
-    T30_ERR_DCNNORTNRX,     /* Unexpected DCN after requested retransmission */
+    T30_ERR_ECMPHDRX,       /*! Invalid ECM response received from transmitter */
+    T30_ERR_GOTDCSRX,       /*! DCS received while waiting for DTC */
+    T30_ERR_INVALCMDRX,     /*! Unexpected command after page received */
+    T30_ERR_NOCARRIERRX,    /*! Carrier lost during fax receive */
+    T30_ERR_NOEOLRX,        /*! Timed out while waiting for EOL (end Of line) */
+    T30_ERR_NOFAXRX,        /*! Timed out while waiting for first line */
+    T30_ERR_T2EXPDCNRX,     /*! Timer T2 expired while waiting for DCN */
+    T30_ERR_T2EXPDRX,       /*! Timer T2 expired while waiting for phase D */
+    T30_ERR_T2EXPFAXRX,     /*! Timer T2 expired while waiting for fax page */
+    T30_ERR_T2EXPMPSRX,     /*! Timer T2 expired while waiting for next fax page */
+    T30_ERR_T2EXPRRRX,      /*! Timer T2 expired while waiting for RR command */
+    T30_ERR_T2EXPRX,        /*! Timer T2 expired while waiting for NSS, DCS or MCF */
+    T30_ERR_DCNWHYRX,       /*! Unexpected DCN while waiting for DCS or DIS */
+    T30_ERR_DCNDATARX,      /*! Unexpected DCN while waiting for image data */
+    T30_ERR_DCNFAXRX,       /*! Unexpected DCN while waiting for EOM, EOP or MPS */
+    T30_ERR_DCNPHDRX,       /*! Unexpected DCN after EOM or MPS sequence */
+    T30_ERR_DCNRRDRX,       /*! Unexpected DCN after RR/RNR sequence */
+    T30_ERR_DCNNORTNRX,     /*! Unexpected DCN after requested retransmission */
 
-    T30_ERR_BADPAGE,        /* TIFF/F page number tag missing */
-    T30_ERR_BADTAG,         /* Incorrect values for TIFF/F tags */
-    T30_ERR_BADTIFFHDR,     /* Bad TIFF/F header - incorrect values in fields */
-    T30_ERR_BADPARM,        /* Invalid value for fax parameter */
-    T30_ERR_BADSTATE,       /* Invalid initial state value specified */
-    T30_ERR_CMDDATA,        /* Last command contained invalid data */
-    T30_ERR_DISCONNECT,     /* Fax call disconnected by the other station */
-    T30_ERR_INVALARG,       /* Illegal argument to function */
-    T30_ERR_INVALFUNC,      /* Illegal call to function */
-    T30_ERR_NODATA,         /* Data requested is not available (NSF, DIS, DCS) */
-    T30_ERR_NOMEM,          /* Cannot allocate memory for more pages */
-    T30_ERR_NOPOLL,         /* Poll not accepted */
-    T30_ERR_NOSTATE,        /* Initial state value not set */
-    T30_ERR_RETRYDCN,       /* Disconnected after permitted retries */
+    T30_ERR_BADPAGE,        /*! TIFF/F page number tag missing */
+    T30_ERR_BADTAG,         /*! Incorrect values for TIFF/F tags */
+    T30_ERR_BADTIFFHDR,     /*! Bad TIFF/F header - incorrect values in fields */
+    T30_ERR_NODATA,         /*! Data requested is not available (NSF, DIS, DCS) */
+    T30_ERR_NOMEM,          /*! Cannot allocate memory for more pages */
+    T30_ERR_NOPOLL,         /*! Poll not accepted */
+    T30_ERR_RETRYDCN,       /*! Disconnected after permitted retries */
+    T30_ERR_CALLDROPPED     /*! The call dropped prematurely */
 };
 
 /*!
@@ -273,6 +277,82 @@ enum
     T30_MODEM_DONE
 };
 
+enum
+{
+    T30_SUPPORT_V27TER = 0x01,
+    T30_SUPPORT_V29 = 0x02,
+    T30_SUPPORT_V17 = 0x04,
+    T30_SUPPORT_V34 = 0x08,
+    T30_SUPPORT_IAF = 0x10,
+};
+
+enum
+{
+    T30_SUPPORT_NO_COMPRESSION = 0x01,
+    T30_SUPPORT_T4_1D_COMPRESSION = 0x02,
+    T30_SUPPORT_T4_2D_COMPRESSION = 0x04,
+    T30_SUPPORT_T6_COMPRESSION = 0x08,
+    T30_SUPPORT_T85_COMPRESSION = 0x10,     /* Monochrome JBIG */
+    T30_SUPPORT_T43_COMPRESSION = 0x20,     /* Colour JBIG */
+    T30_SUPPORT_T45_COMPRESSION = 0x40      /* Run length colour compression */
+};
+
+enum
+{
+    T30_SUPPORT_STANDARD_RESOLUTION = 0x01,
+    T30_SUPPORT_FINE_RESOLUTION = 0x02,
+    T30_SUPPORT_SUPERFINE_RESOLUTION = 0x04,
+
+    T30_SUPPORT_R4_RESOLUTION = 0x10000,
+    T30_SUPPORT_R8_RESOLUTION = 0x20000,
+    T30_SUPPORT_R16_RESOLUTION = 0x40000,
+
+    T30_SUPPORT_300_300_RESOLUTION = 0x100000,
+    T30_SUPPORT_400_400_RESOLUTION = 0x200000,
+    T30_SUPPORT_600_600_RESOLUTION = 0x400000,
+    T30_SUPPORT_1200_1200_RESOLUTION = 0x800000,
+    T30_SUPPORT_300_600_RESOLUTION = 0x1000000,
+    T30_SUPPORT_400_800_RESOLUTION = 0x2000000,
+    T30_SUPPORT_600_1200_RESOLUTION = 0x4000000
+};
+
+enum
+{
+    T30_SUPPORT_215MM_WIDTH = 0x01,
+    T30_SUPPORT_255MM_WIDTH = 0x02,
+    T30_SUPPORT_303MM_WIDTH = 0x04,
+
+    T30_SUPPORT_UNLIMITED_LENGTH = 0x10000,
+    T30_SUPPORT_A4_LENGTH = 0x20000,
+    T30_SUPPORT_B4_LENGTH = 0x40000,
+    T30_SUPPORT_US_LETTER_LENGTH = 0x80000,
+    T30_SUPPORT_US_LEGAL_LENGTH = 0x100000
+};
+
+enum
+{
+    T30_SUPPORT_SEP = 0x01,
+    T30_SUPPORT_PSA = 0x02
+};
+
+enum
+{
+    T30_IAF_MODE_T37 = 0x01,
+    T30_IAF_MODE_T38 = 0x02,
+    T30_IAF_MODE_FLOW_CONTROL = 0x04,
+    /*! Continuous flow mode means data is sent as fast as possible, usually across
+        the Internet, where speed is not constrained by a PSTN modem. */
+    T30_IAF_MODE_CONTINUOUS_FLOW = 0x08,
+    /*! No TCF means TCF is not exchanged. The end points must sort out usable speed
+        issues locally. */
+    T30_IAF_MODE_NO_TCF = 0x10,
+    /*! No fill bits means do not insert fill bits, even if the T.30 messages request
+        them. */
+    T30_IAF_MODE_NO_FILL_BITS = 0x20,
+    /*! No indicators means do not send indicator messages when using T.38. */
+    T30_IAF_MODE_NO_INDICATORS = 0x40
+};
+
 /*!
     T.30 FAX channel descriptor. This defines the state of a single working
     instance of a T.30 FAX channel.
@@ -287,20 +367,31 @@ struct t30_state_s
     int calling_party;
 
     /*! \brief The local identifier string. */
-    char local_ident[21];
+    char local_ident[T30_MAX_IDENT_LEN];
     /*! \brief The identifier string supplied by the remote FAX machine. */
-    char far_ident[21];
+    char far_ident[T30_MAX_IDENT_LEN];
+    /*! \brief The sub-address string to be sent to the remote FAX machine. */
+    char local_sub_address[T30_MAX_IDENT_LEN];
     /*! \brief The sub-address string supplied by the remote FAX machine. */
-    char sub_address[21];
+    char far_sub_address[T30_MAX_IDENT_LEN];
+    /*! \brief The selective polling sub-address supplied by the remote FAX machine. */
+    char sep_address[T30_MAX_IDENT_LEN];
+    /*! \brief The polled sub-address supplied by the remote FAX machine. */
+    char psa_address[T30_MAX_IDENT_LEN];
     /*! \brief A password to be associated with the T.30 context. */
-    char password[21];
+    char local_password[T30_MAX_IDENT_LEN];
+    /*! \brief A password expected from the far end. */
+    char far_password[T30_MAX_IDENT_LEN];
+    /*! \brief The text which will be used in FAX page header. No text results
+               in no header line. */
+    char header_info[50 + 1];
     /*! \brief The country of origin of the remote machine, if known, else NULL. */
     const char *country;
     /*! \brief The vendor of the remote machine, if known, else NULL. */
     const char *vendor;
     /*! \brief The model of the remote machine, if known, else NULL. */
     const char *model;
-    uint8_t local_nsf[100];
+    uint8_t local_nsf[T30_MAX_LOCAL_NSF_LEN];
     int local_nsf_len;
 
     /*! \brief A pointer to a callback routine to be called when phase B events
@@ -336,14 +427,16 @@ struct t30_state_s
     int phase;
     int next_phase;
     int state;
-    int mode;
-    
-    uint8_t dcs_frame[22];
+    int step;
+
+    uint8_t dcs_frame[T30_MAX_DIS_DTC_DCS_LEN];
     int dcs_len;
-    uint8_t dis_dtc_frame[22];
+    uint8_t dis_dtc_frame[T30_MAX_DIS_DTC_DCS_LEN];
     int dis_dtc_len;
-    /*! \brief A valid DIS has been received from the far end. */
+    /*! \brief TRUE if a valid DIS has been received from the far end. */
     int dis_received;
+    /*! \brief TRUE if a valid passwrod has been received from the far end. */
+    int far_password_ok;
 
     /*! \brief A flag to indicate a message is in progress. */
     int in_message;
@@ -358,8 +451,12 @@ struct t30_state_s
 
     /*! \brief The current fallback step for the fast message transfer modem. */
     int current_fallback;
+    /*! \brief The subset of supported modems allowed at the current time, allowing for negotiation. */
+    int current_permitted_modems;
     /*! \brief TRUE if a carrier is present. Otherwise FALSE. */
     int rx_signal_present;
+    /*! \brief TRUE if a modem has trained correctly. */
+    int rx_trained;
     int current_rx_type;
     int current_tx_type;
 
@@ -377,23 +474,46 @@ struct t30_state_s
     int timer_t3;
     /*! \brief This is only used in error correcting mode. */
     int timer_t5;
+    /*! \brief This is only used in full duplex (e.g. ISDN) modes. */
+    int timer_t6;
+    /*! \brief This is only used in full duplex (e.g. ISDN) modes. */
+    int timer_t7;
+    /*! \brief This is only used in full duplex (e.g. ISDN) modes. */
+    int timer_t8;
 
     int far_end_detected;
 
     int local_interrupt_pending;
     int crp_enabled;
-    int error_correcting;
     int line_encoding;
     int min_row_bits;
+    int x_resolution;
     int y_resolution;
     int image_width;
     int retries;
     int error_correcting_mode;
     int ppr_count;
+    int receiver_not_ready_count;
     int octets_per_ecm_frame;
-    uint8_t ecm_data[256][256];
-    uint8_t ecm_status[256];
-    int highest_ecm_frame;
+    uint8_t ecm_data[256][260];
+    int16_t ecm_len[256];
+    uint8_t ecm_frame_map[3 + 32];
+    
+    /*! \brief The current page number in ECM mode */
+    int ecm_page;
+
+    /*! \brief The current block number in ECM mode */
+    int ecm_block;
+    
+    /*! \brief The number of frames in the current block number in ECM mode */
+    int ecm_frames;
+
+    /*! \brief The number of frames in the current burst of image transmission in ECM mode */
+    int ecm_frames_this_burst;
+    int ecm_current_frame;
+    int ecm_at_page_end;
+    int next_tx_step;
+    int next_rx_step;
     char rx_file[256];
     int rx_stop_page;
     char tx_file[256];
@@ -401,6 +521,16 @@ struct t30_state_s
     int tx_stop_page;
     int current_status;
     int iaf;
+    int supported_modems;
+    int supported_compressions;
+    int supported_resolutions;
+    int supported_image_sizes;
+    int supported_polling_features;
+    int ecm_allowed;
+    
+    int last_pps_fcf2;
+    int ecm_first_bad_frame;
+
     /*! \brief Error and flow logging control */
     logging_state_t logging;
 };
@@ -409,6 +539,8 @@ typedef struct
 {
     /*! \brief The current bit rate for image transfer. */
     int bit_rate;
+    /*! \brief TRUE if error correcting mode is used. */
+    int error_correcting_mode;
     /*! \brief The number of pages transferred so far. */
     int pages_transferred;
     /*! \brief The number of horizontal pixels in the most recent page. */
@@ -431,8 +563,9 @@ typedef struct
     int current_status;
 } t30_stats_t;
 
-#ifdef __cplusplus
-extern "C" {
+#if defined(__cplusplus)
+extern "C"
+{
 #endif
 
 /*! Initialise a T.30 context.
@@ -440,10 +573,15 @@ extern "C" {
     \param s The T.30 context.
     \param calling_party TRUE if the context is for a calling party. FALSE if the
            context is for an answering party.
-    \param user_data An opaque pointer which is associated with the T.30 context,
-           and supplied in callbacks.
     \return 0 for OK, else -1. */
-int t30_init(t30_state_t *s, int calling_party, void *user_data);
+int t30_init(t30_state_t *s,
+             int calling_party,
+             t30_set_handler_t *set_rx_type_handler,
+             void *set_rx_type_user_data,
+             t30_set_handler_t *set_tx_type_handler,
+             void *set_tx_type_user_data,
+             t30_send_hdlc_handler_t *send_hdlc_handler,
+             void *send_hdlc_user_data);
 
 /*! Release a T.30 context.
     \brief Release a T.30 context.
@@ -460,17 +598,26 @@ int t30_restart(t30_state_t *s);
     \brief Create and initialise a T.30 context.
     \param calling_party TRUE if the context is for a calling party. FALSE if the
            context is for an answering party.
-    \param user_data An opaque pointer which is associated with the FAX context,
-           and supplied in callbacks.
     \return A pointer to the FAX context, or NULL if one could not be created.
 */
-t30_state_t *t30_create(int calling_party, void *user_data);
+t30_state_t *t30_create(int calling_party,
+                        t30_set_handler_t *set_rx_type_handler,
+                        void *set_rx_type_user_data,
+                        t30_set_handler_t *set_tx_type_handler,
+                        void *set_tx_type_user_data,
+                        t30_send_hdlc_handler_t *send_hdlc_handler,
+                        void *send_hdlc_user_data);
 
 /*! Free a T.30 context.
     \brief Free a T.30 context.
     \param s The T.30 context. */
 void t30_free(t30_state_t *s);
 
+/*! Cleanup a T.30 context if the call terminates.
+    \brief Cleanup a T.30 context if the call terminates.
+    \param s The T.30 context. */
+void t30_terminate(t30_state_t *s);
+
 /*! Return a text name for a T.30 frame type.
     \brief Return a text name for a T.30 frame type.
     \param x The frametype octet.
@@ -502,7 +649,7 @@ void t30_set_iaf_mode(t30_state_t *s, int iaf);
     \param s The T.30 context.
     \param sub_address A pointer to the sub-address.
     \return 0 for OK, else -1. */
-int t30_set_sub_address(t30_state_t *s, const char *sub_address);
+int t30_set_local_sub_address(t30_state_t *s, const char *sub_address);
 
 /*! Set the header information associated with a T.30 context.
     \brief Set the header information associated with a T.30 context.
@@ -621,6 +768,42 @@ void t30_set_rx_file(t30_state_t *s, const char *file, int stop_page);
     \param stop_page The last page to send. -1 for no restriction. */
 void t30_set_tx_file(t30_state_t *s, const char *file, int start_page, int stop_page);
 
+/*! Specify which modem types are supported by a T.30 context.
+    \brief Specify supported modems.
+    \param s The T.30 context.
+    \param supported_modems Bit field list of the supported modems. */
+void t30_set_supported_modems(t30_state_t *s, int supported_modems);
+
+/*! Specify a period of responding with receiver not ready.
+    \brief Specify a period of responding with receiver not ready.
+    \param s The T.30 context.
+    \param count The number of times to report receiver not ready. */
+void t30_set_receiver_not_ready(t30_state_t *s, int count);
+
+/*! Specify which compression types are supported by a T.30 context.
+    \brief Specify supported compression types.
+    \param s The T.30 context.
+    \param supported_compressions Bit field list of the supported compression types. */
+void t30_set_supported_compressions(t30_state_t *s, int supported_compressions);
+
+/*! Specify which resolutions are supported by a T.30 context.
+    \brief Specify supported resolutions.
+    \param s The T.30 context.
+    \param supported_compressions Bit field list of the supported resolutions. */
+void t30_set_supported_resolutions(t30_state_t *s, int supported_resolutions);
+
+/*! Specify which images sizes are supported by a T.30 context.
+    \brief Specify supported image sizes.
+    \param s The T.30 context.
+    \param supported_image_sizes Bit field list of the supported widths and lengths. */
+void t30_set_supported_image_sizes(t30_state_t *s, int supported_image_sizes);
+
+/*! Specify if error correction mode (ECM) is allowed by a T.30 context.
+    \brief Select ECM capability.
+    \param s The T.30 context.
+    \param enabled TRUE for ECM capable, FALSE for not ECM capable. */
+void t30_set_ecm_capability(t30_state_t *s, int enabled);
+
 /*! Request a local interrupt of FAX exchange.
     \brief Request a local interrupt of FAX exchange.
     \param s The T.30 context.
@@ -632,17 +815,51 @@ void t30_local_interrupt_request(t30_state_t *s, int state);
     \param s The T.30 context. */
 void t30_send_complete(void *user_data);
 
+/*! Inform the T.30 engine the current receive has completed. This is
+    only needed to report an unexpected end of the receive operation, as
+    might happen with T.38 dying.
+    \brief Inform the T.30 engine the current receive has completed.
+    \param s The T.30 context. */
+void t30_receive_complete(void *user_data);
+
 /*! Get a bit of received non-ECM image data.
     \brief Get a bit of received non-ECM image data.
     \param user_data An opaque pointer, which must point to the T.30 context.
-    \return bit The next bit to transmit. */
-int t30_non_ecm_getbit(void *user_data);
+    \return The next bit to transmit. */
+int t30_non_ecm_get_bit(void *user_data);
+
+/*! Get a byte of received non-ECM image data.
+    \brief Get a byte of received non-ECM image data.
+    \param user_data An opaque pointer, which must point to the T.30 context.
+    \return The next byte to transmit. */
+int t30_non_ecm_get_byte(void *user_data);
+
+/*! Get a chunk of received non-ECM image data.
+    \brief Get a bit of received non-ECM image data.
+    \param user_data An opaque pointer, which must point to the T.30 context.
+    \param buf The buffer to contain the data.
+    \param max_len The maximum length of the chunk.
+    \return The actual length of the chunk. */
+int t30_non_ecm_get_chunk(void *user_data, uint8_t buf[], int max_len);
 
 /*! Process a bit of received non-ECM image data.
     \brief Process a bit of received non-ECM image data
     \param user_data An opaque pointer, which must point to the T.30 context.
     \param bit The received bit. */
-void t30_non_ecm_putbit(void *user_data, int bit);
+void t30_non_ecm_put_bit(void *user_data, int bit);
+
+/*! Process a byte of received non-ECM image data.
+    \brief Process a byte of received non-ECM image data
+    \param user_data An opaque pointer, which must point to the T.30 context.
+    \param byte The received byte. */
+void t30_non_ecm_put_byte(void *user_data, int byte);
+
+/*! Process a chunk of received non-ECM image data.
+    \brief Process a chunk of received non-ECM image data
+    \param user_data An opaque pointer, which must point to the T.30 context.
+    \param buf The buffer containing the received data.
+    \param len The length of the data in buf. */
+void t30_non_ecm_put_chunk(void *user_data, const uint8_t buf[], int len);
 
 /*! Process a received HDLC frame.
     \brief Process a received HDLC frame.
@@ -658,7 +875,7 @@ void t30_hdlc_accept(void *user_data, int ok, const uint8_t *msg, int len);
     \param samples The time change in 1/8000th second steps. */
 void t30_timer_update(t30_state_t *s, int samples);
 
-#ifdef __cplusplus
+#if defined(__cplusplus)
 }
 #endif
 
