@@ -23,13 +23,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t31.h,v 1.37 2006/10/24 13:45:28 steveu Exp $
+ * $Id: t31.h,v 1.42 2007/04/30 13:13:36 steveu Exp $
  */
 
 /*! \file */
 
-#if !defined(_T31_H_)
-#define _T31_H_
+#if !defined(_SPANDSP_T31_H_)
+#define _SPANDSP_T31_H_
 
 /*! \page t31_page T.31 Class 1 FAX modem protocol handling
 \section t31_page_sec_1 What does it do?
@@ -152,7 +152,7 @@ struct t31_state_s
     int short_train;
     int hdlc_final;
     int data_final;
-    queue_t rx_queue;
+    queue_state_t *rx_queue;
 
     uint8_t hdlc_rx_buf[256];
     int hdlc_rx_len;
@@ -175,8 +175,9 @@ struct t31_state_s
     logging_state_t logging;
 };
 
-#ifdef __cplusplus
-extern "C" {
+#if defined(__cplusplus)
+extern "C"
+{
 #endif
 
 void t31_call_event(t31_state_t *s, int event);
@@ -236,7 +237,7 @@ t31_state_t *t31_init(t31_state_t *s,
     \return 0 for OK */
 int t31_release(t31_state_t *s);
 
-#ifdef __cplusplus
+#if defined(__cplusplus)
 }
 #endif
 
