@@ -1,8 +1,7 @@
 /*
  * SpanDSP - a series of DSP components for telephony
  *
- * tone_detect.h - General telephony tone detection, and specific
- *                 detection of Bell MF, and MFC/R2.
+ * tone_detect.h - General telephony tone detection.
  *
  * Written by Steve Underwood <steveu@coppice.org>
  *
@@ -23,11 +22,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: tone_detect.h,v 1.27 2006/10/24 13:45:28 steveu Exp $
+ * $Id: tone_detect.h,v 1.31 2007/04/08 08:16:18 steveu Exp $
  */
 
-#if !defined(_TONE_DETECT_H_)
-#define _TONE_DETECT_H_
+#if !defined(_SPANDSP_TONE_DETECT_H_)
+#define _SPANDSP_TONE_DETECT_H_
 
 /*!
     Floating point Goertzel filter descriptor.
@@ -50,8 +49,9 @@ typedef struct
     int current_sample;
 } goertzel_state_t;
 
-#ifdef __cplusplus
-extern "C" {
+#if defined(__cplusplus)
+extern "C"
+{
 #endif
 
 /*! \brief Create a descriptor for use with either a Goertzel transform */
@@ -100,7 +100,7 @@ static __inline__ void goertzel_sample(goertzel_state_t *s, int16_t amp)
 }
 /*- End of function --------------------------------------------------------*/
 
-#ifdef __cplusplus
+#if defined(__cplusplus)
 }
 #endif
 
