@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v17rx.h,v 1.27 2006/05/02 14:04:44 steveu Exp $
+ * $Id: v17rx.h,v 1.39 2007/04/10 16:12:20 steveu Exp $
  */
 
 /*! \file */
@@ -67,29 +66,24 @@ samples/second from points within 0.28 degrees of the centre and mid-points of
 each symbol. The output of the filter is multiplied by a complex carrier, generated
 by a DDS. The result is a baseband signal, requiring no further filtering, apart from
 an adaptive equalizer. The baseband signal is fed to a T/2 adaptive equalizer.
-The Gardner algorithm is used to tune the sampling, so the samples fed to
-the equalizer are close to the mid point and edges of each symbol. Initially
+A band edge component maximisation algorithm is used to tune the sampling, so the samples
+fed to the equalizer are close to the mid point and edges of each symbol. Initially
 the algorithm is very lightly damped, to ensure the symbol alignment pulls in
 quickly. Because the sampling rate will not be precisely the same as the
 transmitter's (the spec. says the symbol timing should be within 0.01%), the
-receiver constantly evaluates and corrects this sampling
-throughout its operation. During the symbol timing maintainence phase, the
-algorithm uses a heavily damped Gardner plus integrate and dump approach to
-updates. This heavy damping achieves several things - the Gardner algorithm is
-statistically based, so the statistics must be smoothed; a number of samples
-must be fed to the equalizer buffer before the equalizer output actually
-responds to a step change in the sampling; we need to prevent rapid fluctuations
-in the sampling position, due to the optimal position being close to a boundary.
+receiver constantly evaluates and corrects this sampling throughout its
+operation. During the symbol timing maintainence phase, the algorithm uses
+a heavier damping.
 
 The carrier is specified as 1800Hz +- 1Hz at the transmitter, and 1800 +-7Hz at
 the receiver. The receive carrier would only be this inaccurate if the link
 includes FDM sections. These are being phased out, but the design must still
 allow for the worst case. Using an initial 1800Hz signal for demodulation gives
 a worst case rotation rate for the constellation of about one degree per symbol.
-Once the Gardner algorithm has been given time to lock to the symbol timing of
-the initial alternating pattern, the phase of the demodulated signal is recorded
-on two successive symbols - once for each of the constellation positions. The
-receiver then tracks the symbol alternations, until a large phase jump occurs.
+Once the symbol timing synchronisation algorithm has been given time to lock to the
+symbol timing of the initial alternating pattern, the phase of the demodulated signal
+is recorded on two successive symbols - once for each of the constellation positions.
+The receiver then tracks the symbol alternations, until a large phase jump occurs.
 This signifies the start of the next phase of the training sequence. At this
 point the total phase shift between the original recorded symbol phase, and the
 symbol phase just before the phase jump occurred is used to provide a coarse
@@ -219,13 +213,18 @@ TCM absolutely transformed the phone line modem business.
 
 /* Target length for the equalizer is about 63 taps, to deal with the worst stuff
    in V.56bis. */
-#define V17_EQUALIZER_PRE_LEN   7  /* this much before the real event */
-#define V17_EQUALIZER_POST_LEN  7  /* this much after the real event */
-#define V17_EQUALIZER_MASK      63 /* one less than a power of 2 >= (V17_EQUALIZER_PRE_LEN + 1 + V17_EQUALIZER_POST_LEN) */
+#define V17_EQUALIZER_PRE_LEN       7  /* this much before the real event */
+#define V17_EQUALIZER_POST_LEN      7  /* this much after the real event */
+#define V17_EQUALIZER_MASK          63 /* one less than a power of 2 >= (V17_EQUALIZER_PRE_LEN + 1 + V17_EQUALIZER_POST_LEN) */
 
-#define V17RX_FILTER_STEPS      27
+#define V17_RX_FILTER_STEPS         27
 
-#define V17_TRELLIS_DEPTH       16
+/* We can store more trellis depth that we look back over, so that we can push out a group
+   of symbols in one go, giving greater processing efficiency, at the expense of a bit more
+   latency through the modem. */
+/* Right now we don't take advantage of this optimisation. */
+#define V17_TRELLIS_STORAGE_DEPTH   16
+#define V17_TRELLIS_LOOKBACK_DEPTH  16
 
 /*!
     V.17 modem receive side descriptor. This defines the working state for a
@@ -247,7 +246,7 @@ typedef struct
     void *qam_user_data;
 
     /*! \brief The route raised cosine (RRC) pulse shaping filter buffer. */
-    float rrc_filter[2*V17RX_FILTER_STEPS];
+    float rrc_filter[2*V17_RX_FILTER_STEPS];
     /*! \brief Current offset into the RRC pulse shaping filter buffer. */
     int rrc_filter_step;
 
@@ -257,11 +256,14 @@ typedef struct
     unsigned int scramble_reg;
     /*! \brief TRUE if the short training sequence is to be used. */
     int short_train;
-    int in_training;
+    /*! \brief The section of the training data we are currently in. */
+    int training_stage;
     int training_count;
     float training_error;
-    int carrier_present;
+    /*! \brief The value of the last signal sample, using the a simple HPF for signal power estimation. */
     int16_t last_sample;
+    /*! \brief >0 if a signal above the minimum is present. It may or may not be a V.17 signal. */
+    int signal_present;
 
     /*! \brief The current phase of the carrier (i.e. the DDS parameter). */
     uint32_t carrier_phase;
@@ -281,29 +283,30 @@ typedef struct
 
     float eq_delta;
     /*! \brief The adaptive equalizer coefficients */
-    complex_t eq_coeff[V17_EQUALIZER_PRE_LEN + 1 + V17_EQUALIZER_POST_LEN];
-    complex_t eq_coeff_save[V17_EQUALIZER_PRE_LEN + 1 + V17_EQUALIZER_POST_LEN];
-    complex_t eq_buf[V17_EQUALIZER_MASK + 1];
+    complexf_t eq_coeff[V17_EQUALIZER_PRE_LEN + 1 + V17_EQUALIZER_POST_LEN];
+    complexf_t eq_coeff_save[V17_EQUALIZER_PRE_LEN + 1 + V17_EQUALIZER_POST_LEN];
+    complexf_t eq_buf[V17_EQUALIZER_MASK + 1];
     /*! \brief Current offset into equalizer buffer. */
     int eq_step;
     int eq_put_step;
 
-    /*! \brief Integration variable for damping the Gardner algorithm tests. */
-    int gardner_integrate;
-    /*! \brief Current step size of Gardner algorithm integration. */
-    int gardner_step;
-    /*! \brief The total gardner timing correction, since the carrier came up.
+    /*! \brief The current half of the baud. */
+    int baud_half;
+    /*! \brief Band edge symbol sync. filter state. */
+    float symbol_sync_low[2];
+    float symbol_sync_high[2];
+    float symbol_sync_dc_filter[2];
+    float baud_phase;
+    /*! \brief The total symbol timing correction since the carrier came up.
                This is only for performance analysis purposes. */
-    int gardner_total_correction;
-    /*! \brief The current fractional phase of the baud timing. */
-    int baud_phase;
+    int total_baud_timing_correction;
 
     /*! \brief Starting phase angles for the coarse carrier aquisition step. */
     int32_t start_angles[2];
     /*! \brief History list of phase angles for the coarse carrier aquisition step. */
     int32_t angles[16];
     /*! \brief A pointer to the current constellation. */
-    const complex_t *constellation;
+    const complexf_t *constellation;
     /*! \brief A pointer to the current space map. There is a space map for
                each trellis state. */
     int space_map;
@@ -313,9 +316,9 @@ typedef struct
     /*! \brief Current pointer to the trellis buffers */
     int trellis_ptr;
     /*! \brief The trellis. */
-    int full_path_to_past_state_locations[V17_TRELLIS_DEPTH][8];
+    int full_path_to_past_state_locations[V17_TRELLIS_STORAGE_DEPTH][8];
     /*! \brief The trellis. */
-    int past_state_locations[V17_TRELLIS_DEPTH][8];
+    int past_state_locations[V17_TRELLIS_STORAGE_DEPTH][8];
     /*! \brief Euclidean distances (actually the sqaures of the distances)
                from the last states of the trellis. */
     float distances[8];
@@ -323,13 +326,14 @@ typedef struct
     logging_state_t logging;
 } v17_rx_state_t;
 
-extern const complex_t v17_14400_constellation[128];
-extern const complex_t v17_12000_constellation[64];
-extern const complex_t v17_9600_constellation[32];
-extern const complex_t v17_7200_constellation[16];
+extern const complexf_t v17_14400_constellation[128];
+extern const complexf_t v17_12000_constellation[64];
+extern const complexf_t v17_9600_constellation[32];
+extern const complexf_t v17_7200_constellation[16];
 
 #ifdef __cplusplus
-extern "C" {
+extern "C"
+{
 #endif
 
 /*! Initialise a V.17 modem receive context.
@@ -368,14 +372,14 @@ void v17_rx_set_put_bit(v17_rx_state_t *s, put_bit_func_t put_bit, void *user_da
     \param amp The audio sample buffer.
     \param len The number of samples in the buffer.
 */
-void v17_rx(v17_rx_state_t *s, const int16_t *amp, int len);
+void v17_rx(v17_rx_state_t *s, const int16_t amp[], int len);
 
 /*! Get a snapshot of the current equalizer coefficients.
     \brief Get a snapshot of the current equalizer coefficients.
     \param s The modem context.
     \param coeffs The vector of complex coefficients.
     \return The number of coefficients in the vector. */
-int v17_rx_equalizer_state(v17_rx_state_t *s, complex_t **coeffs);
+int v17_rx_equalizer_state(v17_rx_state_t *s, complexf_t **coeffs);
 
 /*! Get the current received carrier frequency.
     \param s The modem context.
