@@ -22,13 +22,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v27ter_tx.h,v 1.25 2006/10/24 13:45:28 steveu Exp $
+ * $Id: v27ter_tx.h,v 1.28 2007/04/08 08:16:18 steveu Exp $
  */
 
 /*! \file */
 
-#if !defined(_V27TER_TX_H_)
-#define _V27TER_TX_H_
+#if !defined(_SPANDSP_V27TER_TX_H_)
+#define _SPANDSP_V27TER_TX_H_
 
 /*! \page v27ter_tx_page The V.27ter transmitter
 \section v27ter_tx_page_sec_1 What does it do?
@@ -114,8 +114,9 @@ typedef struct
     logging_state_t logging;
 } v27ter_tx_state_t;
 
-#ifdef __cplusplus
-extern "C" {
+#if defined(__cplusplus)
+extern "C"
+{
 #endif
 
 /*! Adjust a V.27ter modem transmit context's power output.
@@ -164,7 +165,7 @@ void v27ter_tx_set_get_bit(v27ter_tx_state_t *s, get_bit_func_t get_bit, void *u
 */
 int v27ter_tx(v27ter_tx_state_t *s, int16_t amp[], int len);
 
-#ifdef __cplusplus
+#if defined(__cplusplus)
 }
 #endif
 
