@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v29rx.h,v 1.41 2006/10/24 13:45:28 steveu Exp $
+ * $Id: v29rx.h,v 1.45 2007/04/10 16:12:20 steveu Exp $
  */
 
 /*! \file */
@@ -158,12 +158,16 @@ typedef struct
     unsigned int scramble_reg;
     /*! \brief The register for the training scrambler. */
     uint8_t training_scramble_reg;
-    int in_training;
+    /*! \brief The section of the training data we are currently in. */
+    int training_stage;
     int training_cd;
     int training_count;
     float training_error;
-    int carrier_present;
+    /*! \brief The value of the last signal sample, using the a simple HPF for signal power estimation. */
     int16_t last_sample;
+    /*! \brief >0 if a signal above the minimum is present. It may or may not be a V.29 signal. */
+    int signal_present;
+
     /*! \brief TRUE if the previous trained values are to be reused. */
     int old_train;
 
@@ -214,7 +218,8 @@ typedef struct
 } v29_rx_state_t;
 
 #ifdef __cplusplus
-extern "C" {
+extern "C"
+{
 #endif
 
 /*! Initialise a V.29 modem receive context.
