@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v42.h,v 1.17 2006/10/24 13:22:02 steveu Exp $
+ * $Id: v42.h,v 1.22 2007/04/30 13:13:36 steveu Exp $
  */
 
 /*! \page v42_page V.42 modem error correction
@@ -35,8 +35,8 @@ far modem supports V.42 is also defined.
 \section v42_page_sec_2 How does it work?
 */
 
-#if !defined(_V42_H_)
-#define _V42_H_
+#if !defined(_SPANDSP_V42_H_)
+#define _SPANDSP_V42_H_
 
 enum
 {
@@ -111,7 +111,7 @@ typedef struct
     lapm_frame_queue_t *txqueue;
     lapm_frame_queue_t *tx_next;
     lapm_frame_queue_t *tx_last;
-    queue_t tx_queue;
+    queue_state_t *tx_queue;
     
     span_sched_state_t sched;
     /*! \brief Error and flow logging control */
@@ -154,8 +154,9 @@ typedef struct
 /*! Log state machine changes */
 #define LAPM_DEBUG_LAPM_STATE 	    (1 << 2)
 
-#ifdef __cplusplus
-extern "C" {
+#if defined(__cplusplus)
+extern "C"
+{
 #endif
 
 const char *lapm_status_to_str(int status);
@@ -223,7 +224,7 @@ void v42_restart(v42_state_t *s);
     \return 0 if OK */
 int v42_release(v42_state_t *s);
 
-#ifdef __cplusplus
+#if defined(__cplusplus)
 }
 #endif
 
