@@ -26,7 +26,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t31.c,v 1.80 2006/10/24 13:45:26 steveu Exp $
+ * $Id: t31.c,v 1.88 2007/04/30 13:13:36 steveu Exp $
  */
 
 /*! \file */
@@ -45,7 +45,12 @@
 #include <memory.h>
 #include <string.h>
 #include <ctype.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <assert.h>
 #include <tiffio.h>
 
@@ -77,8 +82,11 @@
 #include "spandsp/t31.h"
 
 #define MS_PER_TX_CHUNK             30
-#define INDICATOR_TX_COUNT          4
+#define INDICATOR_TX_COUNT          3
+#define DATA_TX_COUNT               1
+#define DATA_END_TX_COUNT           1
 #define DEFAULT_DTE_TIMEOUT         5
+#define DEFAULT_MIN_POWER           8000
 
 typedef const char *(*at_cmd_service_t)(t31_state_t *s, const char *cmd);
 
@@ -128,6 +136,15 @@ static int dummy_rx(void *s, const int16_t amp[], int len);
 static int silence_rx(void *user_data, const int16_t amp[], int len);
 static int cng_rx(void *user_data, const int16_t amp[], int len);
 
+static void bit_reverse_chunk(uint8_t to[], const uint8_t from[], int len)
+{
+    int i;
+
+    for (i = 0;  i < len;  i++)
+        to[i] = bit_reverse8(from[i]);
+}
+/*- End of function --------------------------------------------------------*/
+
 static __inline__ void t31_set_at_rx_mode(t31_state_t *s, int new_mode)
 {
     s->at_state.at_rx_mode = new_mode;
@@ -199,7 +216,6 @@ static int process_rx_indicator(t38_core_state_t *s, void *user_data, int indica
 static int process_rx_data(t38_core_state_t *s, void *user_data, int data_type, int field_type, const uint8_t *buf, int len)
 {
     t31_state_t *t;
-    int i;
     
     t = (t31_state_t *) user_data;
     switch (data_type)
@@ -226,10 +242,7 @@ static int process_rx_data(t38_core_state_t *s, void *user_data, int data_type,
     {
     case T38_FIELD_HDLC_DATA:
         if (t->hdlc_rx_len + len <= 256 - 2)
-        {
-            for (i = 0;  i < len;  i++)
-                t->hdlc_rx_buf[t->hdlc_rx_len++] = bit_reverse8(buf[i]);
-        }
+            bit_reverse_chunk(t->hdlc_rx_buf + t->hdlc_rx_len, buf, len);
         break;
     case T38_FIELD_HDLC_FCS_OK:
         if (len > 0)
@@ -367,14 +380,14 @@ int t31_t38_send_timeout(t31_state_t *s, int samples)
     case T38_TIMED_STEP_NON_ECM_MODEM_3:
         /* Send a chunk of non-ECM image data */
         //if ((len = get_non_ecm_image_chunk(s, buf, s->octets_per_non_ecm_packet)))
-        //    t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_T4_NON_ECM_DATA, buf, abs(len));
+        //    t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_T4_NON_ECM_DATA, buf, abs(len), DATA_TX_COUNT);
         if (len > 0)
         {
             s->next_send_samples += ms_to_samples(MS_PER_TX_CHUNK);
         }
         else
         {
-            t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_T4_NON_ECM_SIG_END, NULL, 0);
+            t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_T4_NON_ECM_SIG_END, NULL, 0, DATA_END_TX_COUNT);
             t38_core_send_indicator(&s->t38, T38_IND_NO_SIGNAL, INDICATOR_TX_COUNT);
             s->timed_step = T38_TIMED_STEP_NONE;
         }
@@ -388,7 +401,7 @@ int t31_t38_send_timeout(t31_state_t *s, int samples)
     case T38_TIMED_STEP_HDLC_MODEM_2:
         /* Send HDLC octet */
         buf[0] = bit_reverse8(s->hdlc_tx_buf[s->hdlc_tx_ptr++]);
-        t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_HDLC_DATA, buf, 1);
+        t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_HDLC_DATA, buf, 1, DATA_TX_COUNT);
         if (s->hdlc_tx_ptr >= s->hdlc_tx_len)
             s->timed_step = T38_TIMED_STEP_HDLC_MODEM_3;
         s->next_send_samples += ms_to_samples(MS_PER_TX_CHUNK);
@@ -399,7 +412,7 @@ int t31_t38_send_timeout(t31_state_t *s, int samples)
         if (s->hdlc_final)
         {
             s->hdlc_tx_len = 0;
-            t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_HDLC_FCS_OK_SIG_END, NULL, 0);
+            t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_HDLC_FCS_OK_SIG_END, NULL, 0, DATA_END_TX_COUNT);
             s->timed_step = T38_TIMED_STEP_HDLC_MODEM_4;
             at_put_response_code(&s->at_state, AT_RESPONSE_CODE_OK);
             t31_set_at_rx_mode(s, AT_MODE_OFFHOOK_COMMAND);
@@ -408,7 +421,7 @@ int t31_t38_send_timeout(t31_state_t *s, int samples)
         }
         else
         {
-            t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_HDLC_FCS_OK, NULL, 0);
+            t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_HDLC_FCS_OK, NULL, 0, DATA_TX_COUNT);
             s->timed_step = T38_TIMED_STEP_HDLC_MODEM_2;
             at_put_response_code(&s->at_state, AT_RESPONSE_CODE_CONNECT);
         }
@@ -418,7 +431,7 @@ int t31_t38_send_timeout(t31_state_t *s, int samples)
         /* End of HDLC transmission */
         /* We have already sent T38_FIELD_HDLC_FCS_OK_SIG_END. It seems some boxes may not like
            us sending a T38_FIELD_HDLC_SIG_END at this point. Just say there is no signal. */
-        //t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_HDLC_SIG_END, NULL, 0);
+        //t38_core_send_data(&s->t38, s->current_tx_data, T38_FIELD_HDLC_SIG_END, NULL, 0, DATA_END_TX_COUNT);
         t38_core_send_indicator(&s->t38, T38_IND_NO_SIGNAL, INDICATOR_TX_COUNT);
         s->timed_step = T38_TIMED_STEP_NONE;
         break;
@@ -440,11 +453,11 @@ static int t31_modem_control_handler(at_state_t *s, void *user_data, int op, con
     {
     case AT_MODEM_CONTROL_ANSWER:
         t->call_samples = 0;
-        t->fsk_min_power = 0;
+        t->fsk_min_power = DEFAULT_MIN_POWER;
         break;
     case AT_MODEM_CONTROL_CALL:
         t->call_samples = 0;
-        t->fsk_min_power = 0;
+        t->fsk_min_power = DEFAULT_MIN_POWER;
         break;
     case AT_MODEM_CONTROL_ONHOOK:
         if (t->tx_holding)
@@ -463,7 +476,7 @@ static int t31_modem_control_handler(at_state_t *s, void *user_data, int op, con
         restart_modem(t, T31_SILENCE_TX);
         break;
     case AT_MODEM_CONTROL_RESTART:
-        restart_modem(t, (intptr_t) num);
+        restart_modem(t, (int) (intptr_t) num);
         return 0;
     case AT_MODEM_CONTROL_DTE_TIMEOUT:
         if (num)
@@ -643,14 +656,14 @@ static void hdlc_accept(void *user_data, int ok, const uint8_t *msg, int len)
                     else
                     {
                         at_put_response_code(&s->at_state, AT_RESPONSE_CODE_NO_CARRIER);
-                        t31_set_at_rx_mode(s, AT_MODE_OFFHOOK_COMMAND);
                     }
                     s->at_state.dte_is_waiting = FALSE;
+                    t31_set_at_rx_mode(s, AT_MODE_OFFHOOK_COMMAND);
                 }
                 else
                 {
                     buf[0] = AT_RESPONSE_CODE_NO_CARRIER;
-                    queue_write_msg(&(s->rx_queue), buf, 1);
+                    queue_write_msg(s->rx_queue, buf, 1);
                 }
             }
             s->at_state.rx_signal_present = FALSE;
@@ -692,9 +705,17 @@ static void hdlc_accept(void *user_data, int ok, const uint8_t *msg, int len)
             {
                 if (!s->rx_message_received)
                 {
-                    /* Report CONNECT as soon as possible to avoid a timeout. */
-                    at_put_response_code(&s->at_state, AT_RESPONSE_CODE_CONNECT);
-                    s->rx_message_received = TRUE;
+                    if (s->at_state.dte_is_waiting)
+                    {
+                        /* Report CONNECT as soon as possible to avoid a timeout. */
+                        at_put_response_code(&s->at_state, AT_RESPONSE_CODE_CONNECT);
+                        s->rx_message_received = TRUE;
+                    }
+                    else
+                    {
+                        buf[0] = AT_RESPONSE_CODE_CONNECT;
+                        queue_write_msg(s->rx_queue, buf, 1);
+                    }
                 }
             }
             /* To avoid detecting echo as signalling in the future we update
@@ -711,7 +732,8 @@ static void hdlc_accept(void *user_data, int ok, const uint8_t *msg, int len)
         }
         return;
     }
-    if (!s->at_state.ok_is_pending)    /* if ok_is_pending then we just ignore whatever comes in */
+    /* If OK is pending then we just ignore whatever comes in */
+    if (!s->at_state.ok_is_pending)
     {
         if (s->at_state.dte_is_waiting)
         {
@@ -748,7 +770,7 @@ static void hdlc_accept(void *user_data, int ok, const uint8_t *msg, int len)
             /* It is safe to look at the two bytes beyond the length of the message,
                and expect to find the FCS there. */
             memcpy(buf + 1, msg, len + 2);
-            queue_write_msg(&(s->rx_queue), buf, len + 3);
+            queue_write_msg(s->rx_queue, buf, len + 3);
         }
     }
     t31_set_at_rx_mode(s, AT_MODE_OFFHOOK_COMMAND);
@@ -775,7 +797,7 @@ static int restart_modem(t31_state_t *s, int new_modem)
     span_log(&s->logging, SPAN_LOG_FLOW, "Restart modem %d\n", new_modem);
     if (s->modem == new_modem)
         return 0;
-    queue_flush(&(s->rx_queue));
+    queue_flush(s->rx_queue);
     s->modem = new_modem;
     s->data_final = FALSE;
     s->at_state.rx_signal_present = FALSE;
@@ -1172,7 +1194,7 @@ static int process_class1_cmd(at_state_t *t, void *user_data, int direction, int
         else
         {
             /* Wait until we have received a specified period of silence. */
-            queue_flush(&(s->rx_queue));
+            queue_flush(s->rx_queue);
             s->silence_awaited = val*80;
             t31_set_at_rx_mode(s, AT_MODE_DELIVERY);
             restart_modem(s, T31_SILENCE_RX);
@@ -1210,9 +1232,9 @@ static int process_class1_cmd(at_state_t *t, void *user_data, int direction, int
             s->rx_message_received = FALSE;
             do
             {
-                if (!queue_empty(&(s->rx_queue)))
+                if (!queue_empty(s->rx_queue))
                 {
-                    len = queue_read_msg(&(s->rx_queue), msg, 256);
+                    len = queue_read_msg(s->rx_queue, msg, 256);
                     if (len > 1)
                     {
                         if (msg[0] == AT_RESPONSE_CODE_OK)
@@ -1524,7 +1546,7 @@ int t31_rx(t31_state_t *s, int16_t amp[], int len)
         /* Clean up any DC influence. */
         power = power_meter_update(&(s->rx_power), amp[i] - s->last_sample);
         s->last_sample = amp[i];
-        if (power > s->silence_threshold_power)
+        if (power > (s->silence_threshold_power + s->fsk_min_power) / 2)
         {
             s->silence_heard = 0;
         }
@@ -1666,7 +1688,7 @@ t31_state_t *t31_init(t31_state_t *s,
     v27ter_tx_init(&(s->v27ter_tx), 4800, FALSE, non_ecm_get_bit, s);
     silence_gen_init(&(s->silence_gen), 0);
     power_meter_init(&(s->rx_power), 4);
-    s->fsk_min_power = 0;
+    s->fsk_min_power = DEFAULT_MIN_POWER;
     s->last_sample = 0;
     s->silence_threshold_power = power_meter_level_dbm0(-43);
     s->at_state.rx_signal_present = FALSE;
@@ -1684,16 +1706,20 @@ t31_state_t *t31_init(t31_state_t *s,
     s->tx_handler = (span_tx_handler_t *) &silence_gen;
     s->tx_user_data = &(s->silence_gen);
 
-    if (queue_create(&(s->rx_queue), 4096, QUEUE_WRITE_ATOMIC | QUEUE_READ_ATOMIC) < 0)
+    if ((s->rx_queue = queue_create(4096, QUEUE_WRITE_ATOMIC | QUEUE_READ_ATOMIC)) == NULL)
         return NULL;
     at_init(&s->at_state, at_tx_handler, at_tx_user_data, t31_modem_control_handler, s);
     at_set_class1_handler(&s->at_state, process_class1_cmd, s);
     s->at_state.dte_inactivity_timeout = DEFAULT_DTE_TIMEOUT;
     if (tx_t38_packet_handler)
     {
-        t38_core_init(&s->t38, process_rx_indicator, process_rx_data, process_rx_missing, (void *) s);
-        s->t38.tx_packet_handler = tx_t38_packet_handler;
-        s->t38.tx_packet_user_data = tx_t38_packet_user_data;
+        t38_core_init(&s->t38,
+                      process_rx_indicator,
+                      process_rx_data,
+                      process_rx_missing,
+                      (void *) s,
+                      tx_t38_packet_handler,
+                      tx_t38_packet_user_data);
     }
     s->t38_mode = FALSE;
     return s;
