@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t4.c,v 1.63 2006/10/24 13:21:59 steveu Exp $
+ * $Id: t4.c,v 1.71 2007/04/05 16:32:08 steveu Exp $
  */
 
 /*
@@ -73,7 +73,12 @@
 #include <time.h>
 #include <memory.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 #include <tiffio.h>
 
 #include "spandsp/telephony.h"
@@ -100,7 +105,7 @@
 #include "faxfont.h"
 
 #if 1
-#define STATE_TRACE(x,...) /**/
+#define STATE_TRACE(...) /**/
 #else
 void STATE_TRACE(char *format, ...)
 {
@@ -150,7 +155,7 @@ typedef struct
     int16_t runlen;         /* run length, in bits */
 } T4_table_entry;
 
-#define isAligned(p,t)  ((((intptr_t)(p)) & (sizeof(t) - 1)) == 0)
+#define is_aligned(p,t)  ((((intptr_t)(p)) & (sizeof(t) - 1)) == 0)
 
 /* Status values returned instead of a run length */
 #define        T4CODE_EOL        -1        /* NB: ACT_EOL - ACT_WRUNT */
@@ -392,6 +397,40 @@ const T4_table_entry t4_black_codes[] =
     { 12, 0x00, T4CODE_INVALID },        /* 0000 0000 0000 */
 };
 
+#if defined(__i386__)  ||  defined(__x86_64__)   
+static __inline__ int run_length(unsigned int bits)
+{
+    return 7 - top_bit(bits);
+}
+/*- End of function --------------------------------------------------------*/
+#else
+static __inline__ int run_length(unsigned int bits)
+{
+    static const uint8_t run_len[256] =
+    {
+        8, 7, 6, 6, 5, 5, 5, 5, 4, 4, 4, 4, 4, 4, 4, 4, /* 0x00 - 0x0f */
+        3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, /* 0x10 - 0x1f */
+        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, /* 0x20 - 0x2f */
+        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, /* 0x30 - 0x3f */
+        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0x40 - 0x4f */
+        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0x50 - 0x5f */
+        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0x60 - 0x6f */
+        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0x70 - 0x7f */
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x80 - 0x8f */
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x90 - 0x9f */
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xa0 - 0xaf */
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xb0 - 0xbf */
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xc0 - 0xcf */
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xd0 - 0xdf */
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xe0 - 0xef */
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xf0 - 0xff */
+    };
+
+    return run_len[bits];
+}
+/*- End of function --------------------------------------------------------*/
+#endif
+
 static __inline__ int flush_bits_to_image_buffer(t4_state_t *s)
 {
     uint8_t *t;
@@ -670,295 +709,341 @@ int t4_rx_end_page(t4_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-int t4_rx_put_bit(t4_state_t *s, int bit)
+static int t4_rx_put_bits(t4_state_t *s, unsigned int bit_string, int quantity)
 {
     int bits;
+    int i;
 
     /* We decompress bit by bit, as the data stream is received. We need to
        scan continuously for EOLs, so we might as well work this way. */
-    s->bits_to_date = (s->bits_to_date >> 1) | ((bit & 1) << 12);
-    if (++s->bits < 13)
+    s->bits_to_date |= (bit_string << s->bits);
+    s->bits += quantity;
+    if (s->bits < 13)
         return FALSE;
     if (!s->first_eol_seen)
     {
         /* Do not let anything through to the decoder, until an EOL arrives. */
-        if ((s->bits_to_date & 0xFFF) != 0x800)
-            return FALSE;
-        s->bits = (s->line_encoding == T4_COMPRESSION_ITU_T4_1D)  ?  1  :  0;
+        while ((s->bits_to_date & 0xFFF) != 0x800)
+        {
+            s->bits_to_date >>= 1;
+            if (--s->bits < 13)
+                return FALSE;
+        }
+        i = (s->line_encoding == T4_COMPRESSION_ITU_T4_1D)  ?  12  :  13;
+        s->bits -= i;
+        s->bits_to_date >>= i;
         s->first_eol_seen = TRUE;
         return FALSE;
     }
     /* Check if the image has already terminated */
     if (s->consecutive_eols >= 5)
         return TRUE;
-    if (s->row_is_2d  &&  s->black_white == 0)
+    while (s->bits >= 13)
     {
-        switch (T4_black_table[s->bits_to_date & 0x1FFF].state)
+        if (s->row_is_2d  &&  s->black_white == 0)
         {
-        case S_EOL:
-            STATE_TRACE("EOL\n");
-            put_eol(s);
-            s->row_is_2d = !(s->bits_to_date & 0x1000);
-            s->bits -= (T4_black_table[s->bits_to_date & 0x1FFF].width + 1);
-            s->its_black = FALSE;
-            s->row_len = 0;
-            break;
-        default:
-            bits = s->bits_to_date & 0x7F;
-            STATE_TRACE("State %d, %d\n",
-                        T4_common_table[bits].state,
-                        T4_common_table[bits].width);
-            switch (T4_common_table[bits].state)
+            switch (T4_black_table[s->bits_to_date & 0x1FFF].state)
             {
-            case S_Pass:
-                STATE_TRACE("Pass\n");
-                if (s->row_len < s->image_width)
-                {
-                    if (s->pa != s->cur_runs)
-                    {
-                        while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
-                        {
-                            s->b1 += s->pb[0] + s->pb[1];
-                            s->pb += 2;
-                        }
-                    }
-                    s->b1 += *s->pb++;
-                    s->run_length += (s->b1 - s->a0);
-                    s->a0 = s->b1;
-                    s->b1 += *s->pb++;
-                }
-                break;
-            case S_Horiz:
-                STATE_TRACE("Horiz\n");
-                s->its_black = ((int) (s->pa - s->cur_runs)) & 1;
-                s->black_white = 2;
-                break;
-            case S_V0:
-                STATE_TRACE("V0 %d %d %d %d\n",
-                            s->a0,
-                            s->b1,
-                            s->image_width,
-                            s->run_length);
-                if (s->row_len < s->image_width)
-                {
-                    if (s->pa != s->cur_runs)
-                    {
-                        while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
-                        {
-                            s->b1 += s->pb[0] + s->pb[1];
-                            s->pb += 2;
-                        }
-                    }
-                    s->run_length += (s->b1 - s->a0);
-                    s->a0 = s->b1;
-                    put_run(s, ((int) (s->pa - s->cur_runs)) & 1);
-                    s->b1 += *s->pb++;
-                }
-                break;
-            case S_VR:
-                STATE_TRACE("VR[%d] %d %d %d %d\n",
-                            T4_common_table[bits].param,
-                            s->a0,
-                            s->b1,
-                            s->image_width,
-                            s->run_length);
-                if (s->row_len < s->image_width)
-                {
-                    if (s->pa != s->cur_runs)
-                    {
-                        while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
-                        {
-                            s->b1 += s->pb[0] + s->pb[1];
-                            s->pb += 2;
-                        }
-                    }
-                    s->run_length += (s->b1 + T4_common_table[bits].param - s->a0);
-                    s->a0 = s->b1 + T4_common_table[bits].param;
-                    put_run(s, ((int) (s->pa - s->cur_runs)) & 1);
-                    s->b1 += *s->pb++;
-                }
-                break;
-            case S_VL:
-                STATE_TRACE("VL[%d] %d %d %d %d\n",
-                            T4_common_table[bits].param,
-                            s->a0,
-                            s->b1,
-                            s->image_width,
-                            s->run_length);
-                if (s->row_len < s->image_width)
-                {
-                    if (s->pa != s->cur_runs)
-                    {
-                        while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
-                        {
-                            s->b1 += s->pb[0] + s->pb[1];
-                            s->pb += 2;
-                        }
-                    }
-                    s->run_length += (s->b1 - T4_common_table[bits].param - s->a0);
-                    s->a0 = s->b1 - T4_common_table[bits].param;
-                    put_run(s, ((int) (s->pa - s->cur_runs)) & 1);
-                    s->b1 -= *--s->pb;
-                }
-                break;
-            case S_Ext:
-                STATE_TRACE("Ext %d 0x%x\n",
-                            ((s->bits_to_date >> T4_common_table[bits].width) & 0x7),
-                            s->bits_to_date);
-                if (s->row_len < s->image_width)
-                    *s->pa++ = s->image_width - s->a0;
-                break;
-            case S_Null:
-                break;
-            default:
-                span_log(&s->logging, SPAN_LOG_WARNING, "Unexpected T.4 state %d\n", T4_common_table[bits].state);
-                break;
-            }
-            s->bits -= T4_common_table[bits].width;
-            break;
-        }
-    }
-    else
-    {
-        if (s->its_black)
-        {
-            bits = s->bits_to_date & 0x1FFF;
-            STATE_TRACE("Black state %d %d\n", T4_black_table[bits].state, T4_black_table[bits].param);
-            switch (T4_black_table[bits].state)
-            {
-            case S_MakeUpB:
-            case S_MakeUp:
-                if (s->row_len < s->image_width)
-                {
-                    s->run_length += T4_black_table[bits].param;
-                    s->a0 += T4_black_table[bits].param;
-                }
-                break;
-            case S_TermB:
-                if (s->row_len < s->image_width)
-                {
-                    s->run_length += T4_black_table[bits].param;
-                    s->a0 += T4_black_table[bits].param;
-                    put_run(s, 1);
-                    if (s->black_white)
-                    {
-                        if (s->black_white == 1)
-                        {
-                            if (s->pa != s->cur_runs)
-                            {
-                                while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
-                                {
-                                    s->b1 += s->pb[0] + s->pb[1];
-                                    s->pb += 2;
-                                }
-                            }
-                        }
-                        s->black_white--;
-                    }
-                }
-                s->its_black = FALSE;
-                break;
             case S_EOL:
                 STATE_TRACE("EOL\n");
-                if (s->row_len == 0)
-                {
-                    if (++s->consecutive_eols >= 5)
-                        return TRUE;
-                }
-                else
-                {
-                    s->consecutive_eols = 0;
-                    put_eol(s);
-                }
-                if (s->line_encoding != T4_COMPRESSION_ITU_T4_1D)
-                {
-                    s->row_is_2d = !(s->bits_to_date & 0x1000);
-                    s->bits--;
-                }
+                put_eol(s);
+                s->row_is_2d = !(s->bits_to_date & 0x1000);
+                i = T4_black_table[s->bits_to_date & 0x1FFF].width + 1;
+                s->bits -= i;
+                s->bits_to_date >>= i;
                 s->its_black = FALSE;
                 s->row_len = 0;
                 break;
             default:
-                /* Bad black */
-                s->black_white = 0;
+                bits = s->bits_to_date & 0x7F;
+                STATE_TRACE("State %d, %d\n",
+                            T4_common_table[bits].state,
+                            T4_common_table[bits].width);
+                switch (T4_common_table[bits].state)
+                {
+                case S_Pass:
+                    STATE_TRACE("Pass\n");
+                    if (s->row_len < s->image_width)
+                    {
+                        if (s->pa != s->cur_runs)
+                        {
+                            while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
+                            {
+                                s->b1 += s->pb[0] + s->pb[1];
+                                s->pb += 2;
+                            }
+                        }
+                        s->b1 += *s->pb++;
+                        s->run_length += (s->b1 - s->a0);
+                        s->a0 = s->b1;
+                        s->b1 += *s->pb++;
+                    }
+                    break;
+                case S_Horiz:
+                    STATE_TRACE("Horiz\n");
+                    s->its_black = ((int) (s->pa - s->cur_runs)) & 1;
+                    s->black_white = 2;
+                    break;
+                case S_V0:
+                    STATE_TRACE("V0 %d %d %d %d\n",
+                                s->a0,
+                                s->b1,
+                                s->image_width,
+                                s->run_length);
+                    if (s->row_len < s->image_width)
+                    {
+                        if (s->pa != s->cur_runs)
+                        {
+                            while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
+                            {
+                                s->b1 += s->pb[0] + s->pb[1];
+                                s->pb += 2;
+                            }
+                        }
+                        s->run_length += (s->b1 - s->a0);
+                        s->a0 = s->b1;
+                        put_run(s, ((int) (s->pa - s->cur_runs)) & 1);
+                        s->b1 += *s->pb++;
+                    }
+                    break;
+                case S_VR:
+                    STATE_TRACE("VR[%d] %d %d %d %d\n",
+                                T4_common_table[bits].param,
+                                s->a0,
+                                s->b1,
+                                s->image_width,
+                                s->run_length);
+                    if (s->row_len < s->image_width)
+                    {
+                        if (s->pa != s->cur_runs)
+                        {
+                            while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
+                            {
+                                s->b1 += s->pb[0] + s->pb[1];
+                                s->pb += 2;
+                            }
+                        }
+                        s->run_length += (s->b1 + T4_common_table[bits].param - s->a0);
+                        s->a0 = s->b1 + T4_common_table[bits].param;
+                        put_run(s, ((int) (s->pa - s->cur_runs)) & 1);
+                        s->b1 += *s->pb++;
+                    }
+                    break;
+                case S_VL:
+                    STATE_TRACE("VL[%d] %d %d %d %d\n",
+                                T4_common_table[bits].param,
+                                s->a0,
+                                s->b1,
+                                s->image_width,
+                                s->run_length);
+                    if (s->row_len < s->image_width)
+                    {
+                        if (s->pa != s->cur_runs)
+                        {
+                            while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
+                            {
+                                s->b1 += s->pb[0] + s->pb[1];
+                                s->pb += 2;
+                            }
+                        }
+                        s->run_length += (s->b1 - T4_common_table[bits].param - s->a0);
+                        s->a0 = s->b1 - T4_common_table[bits].param;
+                        put_run(s, ((int) (s->pa - s->cur_runs)) & 1);
+                        s->b1 -= *--s->pb;
+                    }
+                    break;
+                case S_Ext:
+                    STATE_TRACE("Ext %d 0x%x\n",
+                                ((s->bits_to_date >> T4_common_table[bits].width) & 0x7),
+                                s->bits_to_date);
+                    if (s->row_len < s->image_width)
+                        *s->pa++ = s->image_width - s->a0;
+                    break;
+                case S_Null:
+                    break;
+                default:
+                    span_log(&s->logging, SPAN_LOG_WARNING, "Unexpected T.4 state %d\n", T4_common_table[bits].state);
+                    break;
+                }
+                i = T4_common_table[bits].width;
+                s->bits -= i;
+                s->bits_to_date >>= i;
                 break;
             }
-            s->bits -= T4_black_table[bits].width;
         }
         else
         {
-            bits = s->bits_to_date & 0xFFF;
-            STATE_TRACE("White state %d %d\n", T4_white_table[bits].state, T4_white_table[bits].param);
-            switch (T4_white_table[bits].state)
+            if (s->its_black)
             {
-            case S_MakeUpW:
-            case S_MakeUp:
-                if (s->row_len < s->image_width)
+                bits = s->bits_to_date & 0x1FFF;
+                STATE_TRACE("Black state %d %d\n", T4_black_table[bits].state, T4_black_table[bits].param);
+                switch (T4_black_table[bits].state)
                 {
-                    s->run_length += T4_white_table[bits].param;
-                    s->a0 += T4_white_table[bits].param;
-                }
-                break;
-            case S_TermW:
-                if (s->row_len < s->image_width)
-                {
-                    s->run_length += T4_white_table[bits].param;
-                    s->a0 += T4_white_table[bits].param;
-                    put_run(s, 0);
-                    if (s->black_white)
+                case S_MakeUpB:
+                case S_MakeUp:
+                    if (s->row_len < s->image_width)
                     {
-                        if (s->black_white == 1)
+                        s->run_length += T4_black_table[bits].param;
+                        s->a0 += T4_black_table[bits].param;
+                    }
+                    break;
+                case S_TermB:
+                    if (s->row_len < s->image_width)
+                    {
+                        s->run_length += T4_black_table[bits].param;
+                        s->a0 += T4_black_table[bits].param;
+                        put_run(s, 1);
+                        if (s->black_white)
                         {
-                            if (s->pa != s->cur_runs)
+                            if (s->black_white == 1)
                             {
-                                while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
+                                if (s->pa != s->cur_runs)
                                 {
-                                    s->b1 += s->pb[0] + s->pb[1];
-                                    s->pb += 2;
+                                    while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
+                                    {
+                                        s->b1 += s->pb[0] + s->pb[1];
+                                        s->pb += 2;
+                                    }
                                 }
                             }
+                            s->black_white--;
                         }
-                        s->black_white--;
                     }
+                    s->its_black = FALSE;
+                    break;
+                case S_EOL:
+                    STATE_TRACE("EOL\n");
+                    if (s->row_len == 0)
+                    {
+                        if (++s->consecutive_eols >= 5)
+                            return TRUE;
+                    }
+                    else
+                    {
+                        s->consecutive_eols = 0;
+                        put_eol(s);
+                    }
+                    if (s->line_encoding != T4_COMPRESSION_ITU_T4_1D)
+                    {
+                        s->row_is_2d = !(s->bits_to_date & 0x1000);
+                        s->bits--;
+                        s->bits_to_date >>= 1;
+                    }
+                    s->its_black = FALSE;
+                    s->row_len = 0;
+                    break;
+                default:
+                    /* Bad black */
+                    s->black_white = 0;
+                    break;
                 }
-                s->its_black = TRUE;
-                break;
-            case S_EOL:
-                STATE_TRACE("EOL\n");
-                if (s->row_len == 0)
-                {
-                    if (++s->consecutive_eols >= 5)
-                        return TRUE;
-                }
-                else
-                {
-                    s->consecutive_eols = 0;
-                    put_eol(s);
-                }
-                if (s->line_encoding != T4_COMPRESSION_ITU_T4_1D)
-                {
-                    s->row_is_2d = !(s->bits_to_date & 0x1000);
-                    s->bits--;
-                }
-                s->its_black = FALSE;
-                s->row_len = 0;
-                break;
-            default:
-                /* Bad white */
-                s->black_white = 0;
-                break;
+                s->bits -= T4_black_table[bits].width;
+                s->bits_to_date >>= T4_black_table[bits].width;
             }
-            s->bits -= T4_white_table[bits].width;
+            else
+            {
+                bits = s->bits_to_date & 0xFFF;
+                STATE_TRACE("White state %d %d\n", T4_white_table[bits].state, T4_white_table[bits].param);
+                switch (T4_white_table[bits].state)
+                {
+                case S_MakeUpW:
+                case S_MakeUp:
+                    if (s->row_len < s->image_width)
+                    {
+                        s->run_length += T4_white_table[bits].param;
+                        s->a0 += T4_white_table[bits].param;
+                    }
+                    break;
+                case S_TermW:
+                    if (s->row_len < s->image_width)
+                    {
+                        s->run_length += T4_white_table[bits].param;
+                        s->a0 += T4_white_table[bits].param;
+                        put_run(s, 0);
+                        if (s->black_white)
+                        {
+                            if (s->black_white == 1)
+                            {
+                                if (s->pa != s->cur_runs)
+                                {
+                                    while (s->b1 <= s->a0  &&  s->b1 < s->image_width)
+                                    {
+                                        s->b1 += s->pb[0] + s->pb[1];
+                                        s->pb += 2;
+                                    }
+                                }
+                            }
+                            s->black_white--;
+                        }
+                    }
+                    s->its_black = TRUE;
+                    break;
+                case S_EOL:
+                    STATE_TRACE("EOL\n");
+                    if (s->row_len == 0)
+                    {
+                        if (++s->consecutive_eols >= 5)
+                            return TRUE;
+                    }
+                    else
+                    {
+                        s->consecutive_eols = 0;
+                        put_eol(s);
+                    }
+                    if (s->line_encoding != T4_COMPRESSION_ITU_T4_1D)
+                    {
+                        s->row_is_2d = !(s->bits_to_date & 0x1000);
+                        s->bits--;
+                        s->bits_to_date >>= 1;
+                    }
+                    s->its_black = FALSE;
+                    s->row_len = 0;
+                    break;
+                default:
+                    /* Bad white */
+                    s->black_white = 0;
+                    break;
+                }
+                s->bits -= T4_white_table[bits].width;
+                s->bits_to_date >>= T4_white_table[bits].width;
+            }
+        }
+
+        if (s->line_encoding == T4_COMPRESSION_ITU_T6  &&  s->row_len >= s->image_width)
+        {
+            /* T.6 has no EOL markers. We sense the end of a line by its length alone. */
+            STATE_TRACE("EOL T.6\n");
+            put_eol(s);
+            s->its_black = FALSE;
+            s->row_len = 0;
         }
     }
+    return FALSE;
+}
+/*- End of function --------------------------------------------------------*/
 
-    if (s->line_encoding == T4_COMPRESSION_ITU_T6  &&  s->row_len >= s->image_width)
+int t4_rx_put_bit(t4_state_t *s, int bit)
+{
+    return t4_rx_put_bits(s, bit & 1, 1);
+}
+/*- End of function --------------------------------------------------------*/
+
+int t4_rx_put_byte(t4_state_t *s, uint8_t byte)
+{
+    return t4_rx_put_bits(s, byte & 0xFF, 8);
+}
+/*- End of function --------------------------------------------------------*/
+
+int t4_rx_put_chunk(t4_state_t *s, const uint8_t buf[], int len)
+{
+    int i;
+    uint8_t byte;
+
+    for (i = 0;  i < len;  i++)
     {
-        /* T.6 has no EOL markers. We sense the end of a line by its length alone. */
-        STATE_TRACE("EOL T.6\n");
-        put_eol(s);
-        s->its_black = FALSE;
-        s->row_len = 0;
+        byte = buf[i];
+        if (t4_rx_put_bits(s, byte & 0xFF, 8))
+            return TRUE;
     }
     return FALSE;
 }
@@ -1253,43 +1338,18 @@ static __inline__ int find0span(uint8_t *bp, int bs, int be)
     int n;
     int span;
     unsigned int *lp;
-#if !defined(__i386__)  &&  !defined(__x86_64__)   
-    static const uint8_t zeroruns[256] =
-    {
-        8, 7, 6, 6, 5, 5, 5, 5, 4, 4, 4, 4, 4, 4, 4, 4, /* 0x00 - 0x0f */
-        3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, /* 0x10 - 0x1f */
-        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, /* 0x20 - 0x2f */
-        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, /* 0x30 - 0x3f */
-        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0x40 - 0x4f */
-        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0x50 - 0x5f */
-        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0x60 - 0x6f */
-        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0x70 - 0x7f */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x80 - 0x8f */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x90 - 0x9f */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xa0 - 0xaf */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xb0 - 0xbf */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xc0 - 0xcf */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xd0 - 0xdf */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xe0 - 0xef */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0xf0 - 0xff */
-    };
-#endif
 
     bits = be - bs;
     bp += bs >> 3;
     /* Check partial byte on LHS. */
     if (bits > 0  &&  (n = (bs & 7)))
     {
-#if defined(__i386__)  ||  defined(__x86_64__)
-        span = 7 - top_bit((*bp << n) & 0xFF);
-#else
-        span = zeroruns[(*bp << n) & 0xFF];
-#endif
-        if (span > 8 - n)       /* table value too generous */
+        span = run_length((*bp << n) & 0xFF);
+        if (span > 8 - n)       /* Value too generous */
             span = 8 - n;
-        if (span > bits)        /* constrain span to bit range */
+        if (span > bits)        /* Constrain span to bit range */
             span = bits;
-        if (n + span < 8)       /* doesn't extend to edge of byte */
+        if (n + span < 8)       /* Doesn't extend to edge of byte */
             return span;
         bits -= span;
         bp++;
@@ -1301,17 +1361,10 @@ static __inline__ int find0span(uint8_t *bp, int bs, int be)
     if (bits >= (int) (2*8*sizeof(unsigned int)))
     {
         /* Align to natural integer boundary and check integers. */
-        while (!isAligned(bp, unsigned int))
+        while (!is_aligned(bp, unsigned int))
         {
-            if (*bp != 0x00)
-            {
-#if defined(__i386__)  ||  defined(__x86_64__)
-                span += (7 - top_bit(*bp));
-#else
-                span += zeroruns[*bp];
-#endif
-                return span;
-            }
+            if (*bp)
+                return span + run_length(*bp);
             span += 8;
             bits -= 8;
             bp++;
@@ -1328,15 +1381,8 @@ static __inline__ int find0span(uint8_t *bp, int bs, int be)
     /* Scan full bytes for all 0's. */
     while (bits >= 8)
     {
-        if (*bp != 0x00)
-        {
-#if defined(__i386__)  ||  defined(__x86_64__)
-            span += (7 - top_bit(*bp));
-#else
-            span += zeroruns[*bp];
-#endif
-            return span;
-        }
+        if (*bp)
+            return span + run_length(*bp);
         span += 8;
         bits -= 8;
         bp++;
@@ -1344,11 +1390,7 @@ static __inline__ int find0span(uint8_t *bp, int bs, int be)
     /* Check partial byte on RHS. */
     if (bits > 0)
     {
-#if defined(__i386__)  ||  defined(__x86_64__)
-        n = (7 - top_bit(*bp));
-#else
-        n = zeroruns[*bp];
-#endif
+        n = run_length(*bp);
         span += ((n > bits)  ?  bits  :  n);
     }
     return span;
@@ -1361,43 +1403,18 @@ static __inline__ int find1span(uint8_t *bp, int bs, int be)
     int n;
     int span;
     unsigned int *lp;
-#if !defined(__i386__)  &&  !defined(__x86_64__)
-    static const uint8_t oneruns[256] =
-    {
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x00 - 0x0f */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x10 - 0x1f */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x20 - 0x2f */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x30 - 0x3f */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x40 - 0x4f */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x50 - 0x5f */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x60 - 0x6f */
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 0x70 - 0x7f */
-        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0x80 - 0x8f */
-        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0x90 - 0x9f */
-        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0xa0 - 0xaf */
-        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, /* 0xb0 - 0xbf */
-        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, /* 0xc0 - 0xcf */
-        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, /* 0xd0 - 0xdf */
-        3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, /* 0xe0 - 0xef */
-        4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 6, 6, 7, 8, /* 0xf0 - 0xff */
-    };
-#endif
 
     bits = be - bs;
     bp += bs >> 3;
     /* Check partial byte on LHS. */
     if (bits > 0  &&  (n = (bs & 7)))
     {
-#if defined(__i386__)  ||  defined(__x86_64__)
-        span = 7 - top_bit(((*bp << n) & 0xFF) ^ 0xFF);
-#else
-        span = oneruns[(*bp << n) & 0xFF];
-#endif
-        if (span > 8 - n)       /* table value too generous */
+        span = run_length(((*bp << n) & 0xFF) ^ 0xFF);
+        if (span > 8 - n)       /* Value too generous */
             span = 8 - n;
-        if (span > bits)        /* constrain span to bit range */
+        if (span > bits)        /* Constrain span to bit range */
             span = bits;
-        if (n + span < 8)       /* doesn't extend to edge of byte */
+        if (n + span < 8)       /* Doesn't extend to edge of byte */
             return span;
         bits -= span;
         bp++;
@@ -1409,17 +1426,10 @@ static __inline__ int find1span(uint8_t *bp, int bs, int be)
     if (bits >= (int) (2*8*sizeof(unsigned int)))
     {
         /* Align to natural integer boundary and check integers. */
-        while (!isAligned(bp, unsigned int))
+        while (!is_aligned(bp, unsigned int))
         {
             if (*bp != 0xFF)
-            {
-#if defined(__i386__)  ||  defined(__x86_64__)
-                span += (7 - top_bit(*bp ^ 0xFF));
-#else
-                span += oneruns[*bp];
-#endif
-                return span;
-            }
+                return span + run_length(*bp ^ 0xFF);
             span += 8;
             bits -= 8;
             bp++;
@@ -1437,14 +1447,7 @@ static __inline__ int find1span(uint8_t *bp, int bs, int be)
     while (bits >= 8)
     {
         if (*bp != 0xFF)
-        {
-#if defined(__i386__)  ||  defined(__x86_64__)
-            span += (7 - top_bit(*bp ^ 0xFF));
-#else
-            span += oneruns[*bp];
-#endif
-            return span;
-        }
+            return span + run_length(*bp ^ 0xFF);
         span += 8;
         bits -= 8;
         bp++;
@@ -1452,11 +1455,7 @@ static __inline__ int find1span(uint8_t *bp, int bs, int be)
     /* Check partial byte on RHS. */
     if (bits > 0)
     {
-#if defined(__i386__)  ||  defined(__x86_64__)
-        n = (7 - top_bit(*bp ^ 0xFF));
-#else
-        n = oneruns[*bp];
-#endif
+        n = run_length(*bp ^ 0xFF);
         span += ((n > bits)  ?  bits  :  n);
     }
     return span;
@@ -1776,7 +1775,7 @@ int t4_tx_start_page(t4_state_t *s)
     int i;
     int pattern;
     int row_bufptr;
-    uint32_t parm;
+    int parm;
     char *t;
     char header[132 + 1];
 
@@ -1823,7 +1822,7 @@ int t4_tx_start_page(t4_state_t *s)
                 s->row_buf[row_bufptr++] = (uint8_t) (pattern >> 8);
                 s->row_buf[row_bufptr++] = (uint8_t) (pattern & 0xFF);
             }
-            for (  ;  row_bufptr <= s->bytes_per_row;  )
+            for (  ;  row_bufptr < s->bytes_per_row;  )
                 s->row_buf[row_bufptr++] = 0;
             switch (s->y_resolution)
             {
@@ -1921,6 +1920,26 @@ int t4_tx_get_bit(t4_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
+int t4_tx_get_byte(t4_state_t *s)
+{
+    if (s->bit_ptr >= s->image_size)
+        return 0x100;
+    return bit_reverse8(s->image_buffer[s->bit_ptr++]);
+}
+/*- End of function --------------------------------------------------------*/
+
+int t4_tx_get_chunk(t4_state_t *s, uint8_t buf[], int max_len)
+{
+    if (s->bit_ptr >= s->image_size)
+        return 0;
+    if (s->bit_ptr + max_len > s->image_size)
+        max_len = s->image_size - s->bit_ptr;
+    bit_reverse(buf, &s->image_buffer[s->bit_ptr], max_len);
+    s->bit_ptr += max_len;
+    return max_len;
+}
+/*- End of function --------------------------------------------------------*/
+
 int t4_tx_check_bit(t4_state_t *s)
 {
     int bit;
