@@ -1,8 +1,7 @@
 /*
  * SpanDSP - a series of DSP components for telephony
  *
- * tone_generate.c - General telephony tone generation, and specific
- *                   generation of Bell MF, MFC/R2, and network supervisory tones.
+ * tone_generate.c - General telephony tone generation.
  *
  * Written by Steve Underwood <steveu@coppice.org>
  *
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: tone_generate.c,v 1.26 2006/10/24 13:45:27 steveu Exp $
+ * $Id: tone_generate.c,v 1.34 2007/03/03 10:40:33 steveu Exp $
  */
 
 /*! \file */
@@ -34,10 +33,16 @@
 
 #include <inttypes.h>
 #include <string.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <time.h>
 #include <fcntl.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 
 #include "spandsp/telephony.h"
 #include "spandsp/dc_restore.h"
@@ -68,10 +73,11 @@ void make_tone_gen_descriptor(tone_gen_descriptor_t *s,
         s->phase_rate[0] = dds_phase_ratef((float) f1);
         s->gain[0] = dds_scaling_dbm0f((float) l1);
     }
-    if (f2 >= 1)
+    s->modulate = (f2 < 0);
+    if (f2)
     {
-        s->phase_rate[1] = dds_phase_ratef((float) f2);
-        s->gain[1] = dds_scaling_dbm0f((float) l2);
+        s->phase_rate[1] = dds_phase_ratef((float) abs(f2));
+        s->gain[1] = (s->modulate)  ?  (float) l2/100.0f  :  dds_scaling_dbm0f((float) l2);
     }
 
     s->duration[0] = d1*8;
@@ -83,21 +89,6 @@ void make_tone_gen_descriptor(tone_gen_descriptor_t *s,
 }
 /*- End of function --------------------------------------------------------*/
 
-void make_tone_descriptor(tone_gen_descriptor_t *desc, cadenced_tone_t *tone)
-{
-    make_tone_gen_descriptor(desc,
-                             tone->f1,
-                             tone->level1,
-                             tone->f2,
-                             tone->level2,
-                             tone->on_time1,
-                             tone->off_time1,
-                             tone->on_time2,
-                             tone->off_time2,
-                             tone->repeat);
-}
-/*- End of function --------------------------------------------------------*/
-
 void tone_gen_init(tone_gen_state_t *s, tone_gen_descriptor_t *t)
 {
     int i;
@@ -106,7 +97,8 @@ void tone_gen_init(tone_gen_state_t *s, tone_gen_descriptor_t *t)
     s->gain[0] = t->gain[0];
     s->phase_rate[1] = t->phase_rate[1];
     s->gain[1] = t->gain[1];
-    
+    s->modulate = t->modulate;
+
     for (i = 0;  i < 4;  i++)
         s->duration[i] = t->duration[i];
     s->repeat = t->repeat;
@@ -124,6 +116,7 @@ int tone_gen(tone_gen_state_t *s, int16_t amp[], int max_samples)
     int samples;
     int limit;
     float xamp;
+    float yamp;
 
     if (s->current_section < 0)
         return  0;
@@ -147,14 +140,20 @@ int tone_gen(tone_gen_state_t *s, int16_t amp[], int max_samples)
             {
                 xamp = 0.0;
                 if (s->phase_rate[0])
-                    xamp += dds_modf(&(s->phase[0]), s->phase_rate[0], s->gain[0], 0);
+                    xamp = dds_modf(&(s->phase[0]), s->phase_rate[0], s->gain[0], 0);
                 if (s->phase_rate[1])
-                    xamp += dds_modf(&(s->phase[1]), s->phase_rate[1], s->gain[1], 0);
+                {
+                    yamp = dds_modf(&(s->phase[1]), s->phase_rate[1], s->gain[1], 0);
+                    if (s->modulate)
+                        xamp *= (1.0f + yamp);
+                    else
+                        xamp += yamp;
+                }
                 /* Saturation of the answer is the right thing at this point.
                    However, we are normally generating well controlled tones,
                    that cannot clip. So, the overhead of doing saturation is
                    a waste of valuable time. */
-                amp[samples] = (int16_t) lrintf(xamp);
+                amp[samples] = (int16_t) rintf(xamp);
             }
         }
         if (s->current_position >= s->duration[s->current_section])
@@ -162,16 +161,13 @@ int tone_gen(tone_gen_state_t *s, int16_t amp[], int max_samples)
             s->current_position = 0;
             if (++s->current_section > 3  ||  s->duration[s->current_section] == 0)
             {
-                if (s->repeat)
-                {
-                    s->current_section = 0;
-                }
-                else
+                if (!s->repeat)
                 {
                     /* Force a quick exit */
                     s->current_section = -1;
                     break;
                 }
+                s->current_section = 0;
             }
         }
     }
