@@ -22,16 +22,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v27ter_rx.c,v 1.72 2006/10/24 13:45:27 steveu Exp $
+ * $Id: v27ter_rx.c,v 1.76 2007/04/10 16:12:20 steveu Exp $
  */
 
 /*! \file */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <inttypes.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 
 #include "spandsp/telephony.h"
 #include "spandsp/logging.h"
@@ -697,15 +706,15 @@ float v27ter_rx_symbol_timing_correction(v27ter_rx_state_t *s)
 
 float v27ter_rx_signal_power(v27ter_rx_state_t *s)
 {
-    return power_meter_dbm0(&s->power);
+    return power_meter_current_dbm0(&s->power);
 }
 /*- End of function --------------------------------------------------------*/
 
 void v27ter_rx_signal_cutoff(v27ter_rx_state_t *s, float cutoff)
 {
     /* The 0.4 factor allows for the gain of the DC blocker */
-    s->carrier_on_power = power_meter_level_dbm0(cutoff + 2.5f)*0.4f;
-    s->carrier_off_power = power_meter_level_dbm0(cutoff - 2.5f)*0.4f;
+    s->carrier_on_power = (int32_t) (power_meter_level_dbm0(cutoff + 2.5f)*0.4f);
+    s->carrier_off_power = (int32_t) (power_meter_level_dbm0(cutoff - 2.5f)*0.4f);
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -819,7 +828,7 @@ static __inline__ int descramble(v27ter_rx_state_t *s, int in_bit)
     }
     else
     {
-        if (s->in_training > TRAINING_STAGE_NORMAL_OPERATION  &&  s->in_training < TRAINING_STAGE_TEST_ONES)
+        if (s->training_stage > TRAINING_STAGE_NORMAL_OPERATION  &&  s->training_stage < TRAINING_STAGE_TEST_ONES)
         {
             s->scrambler_pattern_count = 0;
         }
@@ -832,7 +841,7 @@ static __inline__ int descramble(v27ter_rx_state_t *s, int in_bit)
         }
     }
     s->scramble_reg <<= 1;
-    if (s->in_training > TRAINING_STAGE_NORMAL_OPERATION  &&  s->in_training < TRAINING_STAGE_TEST_ONES)
+    if (s->training_stage > TRAINING_STAGE_NORMAL_OPERATION  &&  s->training_stage < TRAINING_STAGE_TEST_ONES)
         s->scramble_reg |= out_bit;
     else
         s->scramble_reg |= in_bit;
@@ -850,7 +859,7 @@ static __inline__ void put_bit(v27ter_rx_state_t *s, int bit)
 
     /* We need to strip the last part of the training before we let data
        go to the application. */
-    if (s->in_training == TRAINING_STAGE_NORMAL_OPERATION)
+    if (s->training_stage == TRAINING_STAGE_NORMAL_OPERATION)
     {
         s->put_bit(s->user_data, out_bit);
     }
@@ -1006,7 +1015,7 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complexf_t
     z = equalizer_get(s);
 
     //span_log(&s->logging, SPAN_LOG_FLOW, "Equalized symbol - %15.5f %15.5f\n", z.re, z.im);
-    switch (s->in_training)
+    switch (s->training_stage)
     {
     case TRAINING_STAGE_NORMAL_OPERATION:
         decode_baud(s, &z);
@@ -1019,7 +1028,7 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complexf_t
         if (++s->training_count >= 30)
         {
             s->gardner_step = 32;
-            s->in_training = TRAINING_STAGE_LOG_PHASE;
+            s->training_stage = TRAINING_STAGE_LOG_PHASE;
             s->angles[0] =
             s->start_angles[0] = arctan2(z.im, z.re);
         }
@@ -1030,7 +1039,7 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complexf_t
         s->angles[1] =
         s->start_angles[1] = angle;
         s->training_count = 1;
-        s->in_training = TRAINING_STAGE_WAIT_FOR_HOP;
+        s->training_stage = TRAINING_STAGE_WAIT_FOR_HOP;
         break;
     case TRAINING_STAGE_WAIT_FOR_HOP:
         angle = arctan2(z.im, z.re);
@@ -1066,7 +1075,7 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complexf_t
             {
                span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (sequence failed)\n");
                /* Park this modem */
-               s->in_training = TRAINING_STAGE_PARKED;
+               s->training_stage = TRAINING_STAGE_PARKED;
                s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
                break;
             }
@@ -1087,7 +1096,7 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complexf_t
             descramble(s, 1);
             descramble(s, 1);
             s->training_count = 1;
-            s->in_training = TRAINING_STAGE_TRAIN_ON_ABAB;
+            s->training_stage = TRAINING_STAGE_TRAIN_ON_ABAB;
         }
         else if (++s->training_count > V27TER_TRAINING_SEG_3_LEN)
         {
@@ -1095,7 +1104,7 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complexf_t
                of a real training sequence. */
             span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (sequence failed)\n");
             /* Park this modem */
-            s->in_training = TRAINING_STAGE_PARKED;
+            s->training_stage = TRAINING_STAGE_PARKED;
             s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
         }
         break;
@@ -1112,7 +1121,7 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complexf_t
         {
             s->constellation_state = (s->bit_rate == 4800)  ?  4  :  2;
             s->training_count = 0;
-            s->in_training = TRAINING_STAGE_TEST_ONES;
+            s->training_stage = TRAINING_STAGE_TEST_ONES;
             s->carrier_track_i = 400.0;
             s->carrier_track_p = 1000000.0;
         }
@@ -1136,8 +1145,8 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complexf_t
                 s->put_bit(s->user_data, PUTBIT_TRAINING_SUCCEEDED);
                 /* Apply some lag to the carrier off condition, to ensure the last few bits get pushed through
                    the processing. */
-                s->carrier_present = (s->bit_rate == 4800)  ?  90  :  120;
-                s->in_training = TRAINING_STAGE_NORMAL_OPERATION;
+                s->signal_present = (s->bit_rate == 4800)  ?  90  :  120;
+                s->training_stage = TRAINING_STAGE_NORMAL_OPERATION;
                 equalizer_save(s);
                 s->carrier_phase_rate_save = s->carrier_phase_rate;
                 s->agc_scaling_save = s->agc_scaling;
@@ -1147,7 +1156,7 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complexf_t
                 /* Training has failed */
                 span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (constellation mismatch %f)\n", s->training_error);
                 /* Park this modem */
-                s->in_training = TRAINING_STAGE_PARKED;
+                s->training_stage = TRAINING_STAGE_PARKED;
                 s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
             }
         }
@@ -1191,13 +1200,13 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
                a slow to respond DC blocker. Use the most elementary HPF. */
             power = power_meter_update(&(s->power), (amp[i] - s->last_sample) >> 1);
             s->last_sample = amp[i];
-            //span_log(&s->logging, SPAN_LOG_FLOW, "Power = %f\n", power_meter_dbm0(&(s->power)));
-            if (s->carrier_present)
+            //span_log(&s->logging, SPAN_LOG_FLOW, "Power = %f\n", power_meter_current_dbm0(&(s->power)));
+            if (s->signal_present)
             {
                 /* Look for power below turnoff threshold to turn the carrier off */
                 if (power < s->carrier_off_power)
                 {
-                    if (--s->carrier_present <= 0)
+                    if (--s->signal_present <= 0)
                     {
                         /* Count down a short delay, to ensure we push the last
                            few bits through the filters before stopping. */
@@ -1212,10 +1221,10 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
                 /* Look for power exceeding turnon threshold to turn the carrier on */
                 if (power < s->carrier_on_power)
                     continue;
-                s->carrier_present = 1;
+                s->signal_present = 1;
                 s->put_bit(s->user_data, PUTBIT_CARRIER_UP);
             }
-            if (s->in_training != TRAINING_STAGE_PARKED)
+            if (s->training_stage != TRAINING_STAGE_PARKED)
             {
                 /* Only spend effort processing this data if the modem is not
                    parked, after training failure. */
@@ -1225,7 +1234,7 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
                    will fiddle the step to align this with the symbols. */
                 if ((s->eq_put_step -= PULSESHAPER_4800_COEFF_SETS) <= 0)
                 {
-                    if (s->in_training == TRAINING_STAGE_SYMBOL_ACQUISITION)
+                    if (s->training_stage == TRAINING_STAGE_SYMBOL_ACQUISITION)
                     {
                         /* Only AGC during the initial training */
                         s->agc_scaling = (1.0f/PULSESHAPER_4800_GAIN)*1.414f/sqrtf(power);
@@ -1271,13 +1280,13 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
                a slow to respond DC blocker. Use the most elementary HPF. */
             power = power_meter_update(&(s->power), (amp[i] - s->last_sample) >> 1);
             s->last_sample = amp[i];
-            //span_log(&s->logging, SPAN_LOG_FLOW, "Power = %f\n", power_meter_dbm0(&(s->power)));
-            if (s->carrier_present)
+            //span_log(&s->logging, SPAN_LOG_FLOW, "Power = %f\n", power_meter_current_dbm0(&(s->power)));
+            if (s->signal_present)
             {
                 /* Look for power below turnoff threshold to turn the carrier off */
                 if (power < s->carrier_off_power)
                 {
-                    if (--s->carrier_present <= 0)
+                    if (--s->signal_present <= 0)
                     {
                         /* Count down a short delay, to ensure we push the last
                            few bits through the filters before stopping. */
@@ -1292,10 +1301,10 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
                 /* Look for power exceeding turnon threshold to turn the carrier on */
                 if (power < s->carrier_on_power)
                     continue;
-                s->carrier_present = 1;
+                s->signal_present = 1;
                 s->put_bit(s->user_data, PUTBIT_CARRIER_UP);
             }
-            if (s->in_training != TRAINING_STAGE_PARKED)
+            if (s->training_stage != TRAINING_STAGE_PARKED)
             {
                 /* Only spend effort processing this data if the modem is not
                    parked, after training failure. */
@@ -1305,7 +1314,7 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
                    will fiddle the step to align this with the symbols. */
                 if ((s->eq_put_step -= PULSESHAPER_2400_COEFF_SETS) <= 0)
                 {
-                    if (s->in_training == TRAINING_STAGE_SYMBOL_ACQUISITION)
+                    if (s->training_stage == TRAINING_STAGE_SYMBOL_ACQUISITION)
                     {
                         /* Only AGC during the initial training */
                         s->agc_scaling = (1.0f/PULSESHAPER_2400_GAIN)*1.414f/sqrtf(power);
@@ -1360,11 +1369,11 @@ int v27ter_rx_restart(v27ter_rx_state_t *s, int rate, int old_train)
 
     s->scramble_reg = 0x3C;
     s->scrambler_pattern_count = 0;
-    s->in_training = TRAINING_STAGE_SYMBOL_ACQUISITION;
+    s->training_stage = TRAINING_STAGE_SYMBOL_ACQUISITION;
     s->training_bc = 0;
     s->training_count = 0;
     s->training_error = 0.0f;
-    s->carrier_present = 0;
+    s->signal_present = 0;
 
     s->carrier_phase = 0;
     //s->carrier_track_i = 100000.0f;
