@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v29rx.c,v 1.98 2006/10/24 13:45:27 steveu Exp $
+ * $Id: v29rx.c,v 1.102 2007/04/10 16:12:20 steveu Exp $
  */
 
 /*! \file */
@@ -35,7 +35,12 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 
 #include "spandsp/telephony.h"
 #include "spandsp/logging.h"
@@ -1541,14 +1546,14 @@ float v29_rx_symbol_timing_correction(v29_rx_state_t *s)
 void v29_rx_signal_cutoff(v29_rx_state_t *s, float cutoff)
 {
     /* The 0.4 factor allows for the gain of the DC blocker */
-    s->carrier_on_power = power_meter_level_dbm0(cutoff + 2.5f)*0.4f;
-    s->carrier_off_power = power_meter_level_dbm0(cutoff - 2.5f)*0.4f;
+    s->carrier_on_power = (int32_t) (power_meter_level_dbm0(cutoff + 2.5f)*0.4f);
+    s->carrier_off_power = (int32_t) (power_meter_level_dbm0(cutoff - 2.5f)*0.4f);
 }
 /*- End of function --------------------------------------------------------*/
 
 float v29_rx_signal_power(v29_rx_state_t *s)
 {
-    return power_meter_dbm0(&s->power);
+    return power_meter_current_dbm0(&s->power);
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -1701,7 +1706,7 @@ static __inline__ void put_bit(v29_rx_state_t *s, int bit)
 
     /* We need to strip the last part of the training - the test period of all 1s -
        before we let data go to the application. */
-    if (s->in_training == TRAINING_STAGE_NORMAL_OPERATION)
+    if (s->training_stage == TRAINING_STAGE_NORMAL_OPERATION)
     {
         s->put_bit(s->user_data, out_bit);
     }
@@ -1851,15 +1856,12 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
 
     //printf("v = %10.5f %5d - %f %f %d %d\n", v, i, p, s->baud_phase, s->total_baud_timing_correction);
 
-    if (i)
-    {
-        s->eq_put_step += i;
-        s->total_baud_timing_correction += i;
-    }
+    s->eq_put_step += i;
+    s->total_baud_timing_correction += i;
 
     z = equalizer_get(s);
 
-    switch (s->in_training)
+    switch (s->training_stage)
     {
     case TRAINING_STAGE_NORMAL_OPERATION:
         /* Normal operation. */
@@ -1872,7 +1874,7 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
         if (++s->training_count >= 60)
         {
             /* Record the current phase angle */
-            s->in_training = TRAINING_STAGE_LOG_PHASE;
+            s->training_stage = TRAINING_STAGE_LOG_PHASE;
             s->angles[0] =
             s->start_angles[0] = arctan2(z.im, z.re);
         }
@@ -1883,7 +1885,7 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
         s->angles[1] =
         s->start_angles[1] = arctan2(z.im, z.re);
         s->training_count = 1;
-        s->in_training = TRAINING_STAGE_WAIT_FOR_CDCD;
+        s->training_stage = TRAINING_STAGE_WAIT_FOR_CDCD;
         break;
     case TRAINING_STAGE_WAIT_FOR_CDCD:
         target = &z;
@@ -1917,7 +1919,7 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
             {
                span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (sequence failed)\n");
                /* Park this modem */
-               s->in_training = TRAINING_STAGE_PARKED;
+               s->training_stage = TRAINING_STAGE_PARKED;
                s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
                break;
             }
@@ -1931,7 +1933,7 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
             /* We have just seen the first bit of the scrambled sequence, so skip it. */
             bit = scrambled_training_bit(s);
             s->training_count = 1;
-            s->in_training = TRAINING_STAGE_TRAIN_ON_CDCD;
+            s->training_stage = TRAINING_STAGE_TRAIN_ON_CDCD;
             break;
         }
         if (++s->training_count > V29_TRAINING_SEG_2_LEN)
@@ -1940,7 +1942,7 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
                of a real training sequence. */
             span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (sequence failed)\n");
             /* Park this modem */
-            s->in_training = TRAINING_STAGE_PARKED;
+            s->training_stage = TRAINING_STAGE_PARKED;
             s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
         }
         break;
@@ -1954,7 +1956,7 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
         tune_equalizer(s, &z, target);
         if (++s->training_count >= V29_TRAINING_SEG_3_LEN - 48)
         {
-            s->in_training = TRAINING_STAGE_TRAIN_ON_CDCD_AND_TEST;
+            s->training_stage = TRAINING_STAGE_TRAIN_ON_CDCD_AND_TEST;
             s->training_error = 0.0f;
             s->carrier_track_i = 200.0f;
             s->carrier_track_p = 1000000.0f;
@@ -1980,13 +1982,13 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
                 s->training_cd = 0;
                 s->training_error = 0.0f;
                 s->constellation_state = 0;
-                s->in_training = TRAINING_STAGE_TEST_ONES;
+                s->training_stage = TRAINING_STAGE_TEST_ONES;
             }
             else
             {
                 span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (convergence failed)\n");
                 /* Park this modem */
-                s->in_training = TRAINING_STAGE_PARKED;
+                s->training_stage = TRAINING_STAGE_PARKED;
                 s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
             }
         }
@@ -2009,8 +2011,8 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
                 s->put_bit(s->user_data, PUTBIT_TRAINING_SUCCEEDED);
                 /* Apply some lag to the carrier off condition, to ensure the last few bits get pushed through
                    the processing. */
-                s->carrier_present = 60;
-                s->in_training = TRAINING_STAGE_NORMAL_OPERATION;
+                s->signal_present = 60;
+                s->training_stage = TRAINING_STAGE_NORMAL_OPERATION;
                 equalizer_save(s);
                 s->carrier_phase_rate_save = s->carrier_phase_rate;
                 s->agc_scaling_save = s->agc_scaling;
@@ -2021,7 +2023,7 @@ static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
                 span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (constellation mismatch %f)\n", s->training_error);
                 /* Park this modem */
                 s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
-                s->in_training = TRAINING_STAGE_PARKED;
+                s->training_stage = TRAINING_STAGE_PARKED;
             }
         }
         break;
@@ -2060,12 +2062,12 @@ int v29_rx(v29_rx_state_t *s, const int16_t amp[], int len)
            a slow to respond DC blocker. Use the most elementary HPF. */
         power = power_meter_update(&(s->power), (amp[i] - s->last_sample) >> 1);
         s->last_sample = amp[i];
-        if (s->carrier_present)
+        if (s->signal_present)
         {
             /* Look for power below turn-off threshold to turn the carrier off */
             if (power < s->carrier_off_power)
             {
-                if (--s->carrier_present <= 0)
+                if (--s->signal_present <= 0)
                 {
                     /* Count down a short delay, to ensure we push the last
                        few bits through the filters before stopping. */
@@ -2080,10 +2082,10 @@ int v29_rx(v29_rx_state_t *s, const int16_t amp[], int len)
             /* Look for power exceeding turn-on threshold to turn the carrier on */
             if (power < s->carrier_on_power)
                 continue;
-            s->carrier_present = 1;
+            s->signal_present = 1;
             s->put_bit(s->user_data, PUTBIT_CARRIER_UP);
         }
-        if (s->in_training != TRAINING_STAGE_PARKED)
+        if (s->training_stage != TRAINING_STAGE_PARKED)
         {
             /* Only spend effort processing this data if the modem is not
                parked, after training failure. */
@@ -2114,7 +2116,7 @@ int v29_rx(v29_rx_state_t *s, const int16_t amp[], int len)
                will fiddle the step to align this with the symbols. */
             if (s->eq_put_step <= 0)
             {
-                if (s->in_training == TRAINING_STAGE_SYMBOL_ACQUISITION)
+                if (s->training_stage == TRAINING_STAGE_SYMBOL_ACQUISITION)
                 {
                     /* Only AGC during the initial training */
                     s->agc_scaling = (1.0f/PULSESHAPER_GAIN)*5.0f*0.72f/sqrtf(power);
@@ -2175,9 +2177,9 @@ int v29_rx_restart(v29_rx_state_t *s, int rate, int old_train)
 
     s->scramble_reg = 0;
     s->training_scramble_reg = 0x2A;
-    s->in_training = TRAINING_STAGE_SYMBOL_ACQUISITION;
+    s->training_stage = TRAINING_STAGE_SYMBOL_ACQUISITION;
     s->training_count = 0;
-    s->carrier_present = 0;
+    s->signal_present = 0;
     s->old_train = old_train;
 
     s->carrier_phase = 0;
