@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v29tx.c,v 1.56 2006/10/24 13:45:27 steveu Exp $
+ * $Id: v29tx.c,v 1.59 2007/01/03 14:15:36 steveu Exp $
  */
 
 /*! \file */
@@ -35,7 +35,12 @@
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 
 #include "spandsp/telephony.h"
 #include "spandsp/logging.h"
@@ -68,114 +73,114 @@
 static const float pulseshaper[PULSESHAPER_COEFF_SETS][V29_TX_FILTER_STEPS] =
 {
     {
-        -0.0029426223,          /* Filter 0 */
-        -0.0183060118,
-         0.0653192857,
-        -0.1703207714,
-         0.6218069936,
-         0.6218069936,
-        -0.1703207714,
-         0.0653192857,
-        -0.0183060118
+        -0.0029426223f,         /* Filter 0 */
+        -0.0183060118f,
+         0.0653192857f,
+        -0.1703207714f,
+         0.6218069936f,
+         0.6218069936f,
+        -0.1703207714f,
+         0.0653192857f,
+        -0.0183060118f
     },
     {
-         0.0031876922,          /* Filter 1 */
-        -0.0300884145,
-         0.0832744718,
-        -0.1974255221,
-         0.7664229820,
-         0.4670580725,
-        -0.1291107519,
-         0.0424189243,
-        -0.0059810465
+         0.0031876922f,         /* Filter 1 */
+        -0.0300884145f,
+         0.0832744718f,
+        -0.1974255221f,
+         0.7664229820f,
+         0.4670580725f,
+        -0.1291107519f,
+         0.0424189243f,
+        -0.0059810465f
     },
     {
-         0.0097229236,          /* Filter 2 */
-        -0.0394811291,
-         0.0931039664,
-        -0.2043906784,
-         0.8910868760,
-         0.3122713836,
-        -0.0802880559,
-         0.0179050490,
-         0.0052057308
+         0.0097229236f,         /* Filter 2 */
+        -0.0394811291f,
+         0.0931039664f,
+        -0.2043906784f,
+         0.8910868760f,
+         0.3122713836f,
+        -0.0802880559f,
+         0.0179050490f,
+         0.0052057308f
     },
     {
-         0.0156117223,          /* Filter 3 */
-        -0.0447125347,
-         0.0922040267,
-        -0.1862939416,
-         0.9870942864,
-         0.1669790517,
-        -0.0301581072,
-        -0.0051358510,
-         0.0139350286
+         0.0156117223f,         /* Filter 3 */
+        -0.0447125347f,
+         0.0922040267f,
+        -0.1862939416f,
+         0.9870942864f,
+         0.1669790517f,
+        -0.0301581072f,
+        -0.0051358510f,
+         0.0139350286f
     },
     {
-         0.0197702545,          /* Filter 4 */
-        -0.0443470335,
-         0.0789538534,
-        -0.1399184160,
-         1.0476130256,
-         0.0393903028,
-         0.0157339854,
-        -0.0241879599,
-         0.0193774571
+         0.0197702545f,         /* Filter 4 */
+        -0.0443470335f,
+         0.0789538534f,
+        -0.1399184160f,
+         1.0476130256f,
+         0.0393903028f,
+         0.0157339854f,
+        -0.0241879599f,
+         0.0193774571f
     },
     {
-         0.0212455717,          /* Filter 5 */
-        -0.0375307894,
-         0.0530516472,
-        -0.0642195521,
-         1.0682849922,
-        -0.0642195521,
-         0.0530516472,
-        -0.0375307894,
-         0.0212455717
+         0.0212455717f,         /* Filter 5 */
+        -0.0375307894f,
+         0.0530516472f,
+        -0.0642195521f,
+         1.0682849922f,
+        -0.0642195521f,
+         0.0530516472f,
+        -0.0375307894f,
+         0.0212455717f
     },
     {
-         0.0193774571,          /* Filter 6 */
-        -0.0241879599,
-         0.0157339854,
-         0.0393903028,
-         1.0476130256,
-        -0.1399184160,
-         0.0789538534,
-        -0.0443470335,
-         0.0197702545
+         0.0193774571f,         /* Filter 6 */
+        -0.0241879599f,
+         0.0157339854f,
+         0.0393903028f,
+         1.0476130256f,
+        -0.1399184160f,
+         0.0789538534f,
+        -0.0443470335f,
+         0.0197702545f
     },
     {
-         0.0139350286,          /* Filter 7 */
-        -0.0051358510,
-        -0.0301581072,
-         0.1669790517,
-         0.9870942864,
-        -0.1862939416,
-         0.0922040267,
-        -0.0447125347,
-         0.0156117223
+         0.0139350286f,         /* Filter 7 */
+        -0.0051358510f,
+        -0.0301581072f,
+         0.1669790517f,
+         0.9870942864f,
+        -0.1862939416f,
+         0.0922040267f,
+        -0.0447125347f,
+         0.0156117223f
     },
     {
-         0.0052057308,          /* Filter 8 */
-         0.0179050490,
-        -0.0802880559,
-         0.3122713836,
-         0.8910868760,
-        -0.2043906784,
-         0.0931039664,
-        -0.0394811291,
-         0.0097229236
+         0.0052057308f,         /* Filter 8 */
+         0.0179050490f,
+        -0.0802880559f,
+         0.3122713836f,
+         0.8910868760f,
+        -0.2043906784f,
+         0.0931039664f,
+        -0.0394811291f,
+         0.0097229236f
     },
     {
-        -0.0059810465,          /* Filter 9 */
-         0.0424189243,
-        -0.1291107519,
-         0.4670580725,
-         0.7664229820,
-        -0.1974255221,
-         0.0832744718,
-        -0.0300884145,
-         0.0031876922
+        -0.0059810465f,         /* Filter 9 */
+         0.0424189243f,
+        -0.1291107519f,
+         0.4670580725f,
+         0.7664229820f,
+        -0.1974255221f,
+         0.0832744718f,
+        -0.0300884145f,
+         0.0031876922f
     },
 };
 
@@ -351,7 +356,7 @@ int v29_tx(v29_tx_state_t *s, int16_t amp[], int len)
         /* Now create and modulate the carrier */
         z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
         /* Don't bother saturating. We should never clip. */
-        amp[sample] = (int16_t) lrintf((x.re*z.re - x.im*z.im)*s->gain);
+        amp[sample] = (int16_t) rintf((x.re*z.re - x.im*z.im)*s->gain);
     }
     return sample;
 }
