@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v42.c,v 1.29 2006/10/24 13:45:27 steveu Exp $
+ * $Id: v42.c,v 1.32 2007/04/30 13:13:36 steveu Exp $
  */
 
 /* THIS IS A WORK IN PROGRESS. IT IS NOT FINISHED. */
@@ -142,7 +142,7 @@ static void lapm_send_sabme(span_sched_state_t *ss, void *user_data)
     s = (lapm_state_t *) user_data;
     if (s->t401_timer >= 0)
     {
-fprintf(stderr, "Deleting T401 q [%p]\n", s);
+fprintf(stderr, "Deleting T401 q [%p] %d\n", (void *) s, s->t401_timer);
         span_schedule_del(&s->sched, s->t401_timer);
         s->t401_timer = -1;
     }
@@ -157,7 +157,7 @@ fprintf(stderr, "Deleting T401 q [%p]\n", s);
         return;
     }
     /*endif*/
-fprintf(stderr, "Setting T401 a [%p]\n", s);
+fprintf(stderr, "Setting T401 a1 [%p]\n", (void *) s);
     s->t401_timer = span_schedule_event(&s->sched, T_401, lapm_send_sabme, s);
     lapm_init_header(frame, s->we_are_originator);
     frame[1] = 0x7F;
@@ -234,10 +234,10 @@ static void lapm_ack_rx(lapm_state_t *s, int ack)
     {
         span_log(&s->logging, SPAN_LOG_FLOW, "-- Since there was nothing left, stopping timer T_401\n");
         /* Something was ACK'd.  Stop timer T_401. */
-fprintf(stderr, "T401 a is %d [%p]\n", s->t401_timer, s);
+fprintf(stderr, "T401 a2 is %d [%p]\n", s->t401_timer, (void *) s);
         if (s->t401_timer >= 0)
         {
-fprintf(stderr, "Deleting T401 a [%p]\n", s);
+fprintf(stderr, "Deleting T401 a3 [%p] %d\n", (void *) s, s->t401_timer);
             span_schedule_del(&s->sched, s->t401_timer);
             s->t401_timer = -1;
         }
@@ -249,7 +249,7 @@ fprintf(stderr, "Deleting T401 a [%p]\n", s);
         span_log(&s->logging, SPAN_LOG_FLOW, "-- Stopping timer T_403, since we got an ACK\n");
         if (s->t403_timer >= 0)
         {
-fprintf(stderr, "Deleting T403 b\n");
+fprintf(stderr, "Deleting T403 b %d\n", s->t403_timer);
             span_schedule_del(&s->sched, s->t403_timer);
             s->t403_timer = -1;
         }
@@ -265,7 +265,7 @@ fprintf(stderr, "Deleting T403 b\n");
                  s->txqueue->frame[1] >> 1);
         if (s->t401_timer < 0)
         {
-fprintf(stderr, "Setting T401 b [%p]\n", s);
+fprintf(stderr, "Setting T401 b [%p]\n", (void *) s);
             s->t401_timer = span_schedule_event(&s->sched, T_401, t401_expired, s);
         }
         /*endif*/
@@ -313,7 +313,7 @@ static void t401_expired(span_sched_state_t *ss, void *user_data)
     lapm_state_t *s;
     
     s = (lapm_state_t *) user_data;
-fprintf(stderr, "Expiring T401 a [%p]\n", s);
+fprintf(stderr, "Expiring T401 a4 [%p]\n", (void *) s);
     s->t401_timer = -1;
     if (s->txqueue)
     {
@@ -329,7 +329,7 @@ fprintf(stderr, "Expiring T401 a [%p]\n", s);
             span_log(&s->logging, SPAN_LOG_FLOW, "-- Retransmitting %d bytes\n", s->txqueue->len);
             lapm_tx_frame(s, s->txqueue->frame, s->txqueue->len);
             span_log(&s->logging, SPAN_LOG_FLOW, "-- Scheduling retransmission (%d)\n", s->retransmissions);
-fprintf(stderr, "Setting T401 d [%p]\n", s);
+fprintf(stderr, "Setting T401 d [%p]\n", (void *) s);
             s->t401_timer = span_schedule_event(&s->sched, T_401, t401_expired, s);
         }
         else
@@ -379,7 +379,7 @@ const char *lapm_status_to_str(int status)
 
 int lapm_tx(lapm_state_t *s, const void *buf, int len)
 {
-    return queue_write(&s->tx_queue, buf, len);
+    return queue_write(s->tx_queue, buf, len);
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -415,11 +415,7 @@ int lapm_tx_iframe(lapm_state_t *s, const void *buf, int len, int cr)
     }
     /*endif*/
 
-    if (s->peer_is_originator)
-        lapm_init_header(f->frame, cr);
-    else
-        lapm_init_header(f->frame, !cr);
-    /*endif*/
+    lapm_init_header(f->frame, (s->peer_is_originator)  ?  cr  :  !cr);
     f->next = NULL;
     f->len = len + 4;
     f->frame[1] = (uint8_t) (s->next_tx_frame << 1);
@@ -442,7 +438,7 @@ int lapm_tx_iframe(lapm_state_t *s, const void *buf, int len, int cr)
     if (s->t403_timer >= 0)
     {
         span_log(&s->logging, SPAN_LOG_FLOW, "Stopping T_403 timer\n");
-fprintf(stderr, "Deleting T403 c\n");
+fprintf(stderr, "Deleting T403 c %d\n", s->t403_timer);
         span_schedule_del(&s->sched, s->t403_timer);
         s->t403_timer = -1;
     }
@@ -451,7 +447,7 @@ fprintf(stderr, "Deleting T403 c\n");
     {
         span_log(&s->logging, SPAN_LOG_FLOW, "Starting timer T_401\n");
         s->t401_timer = span_schedule_event(&s->sched, T_401, t401_expired, s);
-fprintf(stderr, "Setting T401 e %d [%p]\n", s->t401_timer, s);
+fprintf(stderr, "Setting T401 e %d [%p]\n", s->t401_timer, (void *) s);
     }
     else
     {
@@ -636,16 +632,16 @@ static void lapm_link_up(lapm_state_t *s)
     if (s->status_callback)
         s->status_callback(s->status_callback_user_data, s->state);
     /*endif*/
-    if (!queue_empty(&(s->tx_queue)))
+    if (!queue_empty(s->tx_queue))
     {
-        if ((len = queue_read(&(s->tx_queue), buf, s->n401)) > 0)
+        if ((len = queue_read(s->tx_queue, buf, s->n401)) > 0)
             lapm_tx_iframe(s, buf, len, TRUE);
         /*endif*/
     }
     /*endif*/
     if (s->t401_timer >= 0)
     {
-fprintf(stderr, "Deleting T401 x [%p]\n", s);
+fprintf(stderr, "Deleting T401 x [%p] %d\n", (void *) s, s->t401_timer);
         span_schedule_del(&s->sched, s->t401_timer);
         s->t401_timer = -1;
     }
@@ -680,14 +676,14 @@ void lapm_reset(lapm_state_t *s)
     s->n401 = 128;
     if (s->t401_timer >= 0)
     {
-fprintf(stderr, "Deleting T401 d [%p]\n", s);
+fprintf(stderr, "Deleting T401 d [%p] %d\n", (void *) s, s->t401_timer);
         span_schedule_del(&s->sched, s->t401_timer);
         s->t401_timer = -1;
     }
     /*endif*/
     if (s->t403_timer >= 0)
     {
-fprintf(stderr, "Deleting T403 e\n");
+fprintf(stderr, "Deleting T403 e %d\n", s->t403_timer);
         span_schedule_del(&s->sched, s->t403_timer);
         s->t403_timer = -1;
     }
@@ -913,7 +909,7 @@ fprintf(stderr, "LAPM receive %d %d\n", ok, len);
                     /* Reset t401 timer if it was somehow going */
                     if (s->t401_timer >= 0)
                     {
-fprintf(stderr, "Deleting T401 f [%p]\n", s);
+fprintf(stderr, "Deleting T401 f [%p] %d\n", (void *) s, s->t401_timer);
                         span_schedule_del(&s->sched, s->t401_timer);
                         s->t401_timer = -1;
                     }
@@ -921,7 +917,7 @@ fprintf(stderr, "Deleting T401 f [%p]\n", s);
                     /* Reset and restart t403 timer */
                     if (s->t403_timer >= 0)
                     {
-fprintf(stderr, "Deleting T403 g\n");
+fprintf(stderr, "Deleting T403 g %d\n", s->t403_timer);
                         span_schedule_del(&s->sched, s->t403_timer);
                         s->t403_timer = -1;
                     }
@@ -1102,11 +1098,10 @@ void lapm_restart(lapm_state_t *s)
     /*endif*/
 #endif
     span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
-    span_log_set_protocol(&s->logging, "LAPM");
+    span_log_set_protocol(&s->logging, "LAP.M");
     hdlc_tx_init(&s->hdlc_tx, FALSE, 1, TRUE, lapm_hdlc_underflow, s);
     hdlc_rx_init(&s->hdlc_rx, FALSE, FALSE, 1, lapm_receive, s);
     /* TODO: This is a bodge! */
-fprintf(stderr, "LAPM restart T401 a [%p]\n", s);
     s->t401_timer = -1;
     s->t402_timer = -1;
     s->t403_timer = -1;
@@ -1244,7 +1239,7 @@ static void negotiation_rx_bit(v42_state_t *s, int new_bit)
                 {
                     if (s->t400_timer >= 0)
                     {
-fprintf(stderr, "Deleting T400 h\n");
+fprintf(stderr, "Deleting T400 h %d\n", s->t400_timer);
                         span_schedule_del(&s->lapm.sched, s->t400_timer);
                         s->t400_timer = -1;
                     }
@@ -1312,7 +1307,7 @@ static int v42_support_negotiation_tx_bit(v42_state_t *s)
                 {
                     if (s->t400_timer >= 0)
                     {
-fprintf(stderr, "Deleting T400 i\n");
+fprintf(stderr, "Deleting T400 i %d\n", s->t400_timer);
                         span_schedule_del(&s->lapm.sched, s->t400_timer);
                         s->t400_timer = -1;
                     }
@@ -1424,7 +1419,11 @@ v42_state_t *v42_init(v42_state_t *s, int caller, int detect, v42_frame_handler_
     s->lapm.iframe_receive = frame_handler;
     s->lapm.iframe_receive_user_data = user_data;
     s->lapm.debug |= (LAPM_DEBUG_LAPM_RAW | LAPM_DEBUG_LAPM_DUMP | LAPM_DEBUG_LAPM_STATE);
-    if (queue_create(&(s->lapm.tx_queue), 16384, 0) < 0)
+    s->lapm.t401_timer =
+    s->lapm.t402_timer =
+    s->lapm.t403_timer = -1;
+
+    if ((s->lapm.tx_queue = queue_create(16384, 0)) == NULL)
         return NULL;
     /*endif*/
     span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
