@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: fax_tests.c,v 1.59 2006/10/24 13:45:29 steveu Exp $
+ * $Id: fax_tests.c,v 1.67 2007/05/01 04:26:10 steveu Exp $
  */
 
 /*! \page fax_tests_page FAX tests
@@ -38,7 +38,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <assert.h>
 #include <audiofile.h>
 #include <tiffio.h>
@@ -63,9 +68,12 @@ struct machine_s
     int succeeded;
     char tag[50];
     int error_delay;
+    int total_audio_time;
 } machines[FAX_MACHINES];
 
+int use_receiver_not_ready = FALSE;
 int test_local_interrupt = FALSE;
+int t30_state_to_wreck = -1;
 
 static void phase_b_handler(t30_state_t *s, void *user_data, int result)
 {
@@ -99,6 +107,9 @@ static void phase_d_handler(t30_state_t *s, void *user_data, int result)
     t30_get_far_ident(s, ident);
     printf("%d: Phase D: remote ident '%s'\n", i, ident);
     
+    if (use_receiver_not_ready)
+        t30_set_receiver_not_ready(s, 3);
+
     if (test_local_interrupt)
     {
         if (i == 0)
@@ -188,8 +199,12 @@ int main(int argc, char *argv[])
     int log_audio;
     int use_ecm;
     int use_tep;
+    int use_transmit_on_idle;
     int use_line_hits;
     int polled_mode;
+    int reverse_flow;
+    time_t start_time;
+    time_t end_time;
     char *page_header_info;
 
     log_audio = FALSE;
@@ -200,6 +215,9 @@ int main(int argc, char *argv[])
     use_tep = FALSE;
     polled_mode = FALSE;
     page_header_info = NULL;
+    reverse_flow = FALSE;
+    use_transmit_on_idle = TRUE;
+    use_receiver_not_ready = FALSE;
     for (i = 1;  i < argc;  i++)
     {
         if (strcmp(argv[i], "-e") == 0)
@@ -237,11 +255,26 @@ int main(int argc, char *argv[])
             polled_mode = TRUE;
             continue;
         }
+        if (strcmp(argv[i], "-r") == 0)
+        {
+            reverse_flow = TRUE;
+            continue;
+        }
+        if (strcmp(argv[i], "-R") == 0)
+        {
+            use_receiver_not_ready = TRUE;
+            continue;
+        }
         if (strcmp(argv[i], "-t") == 0)
         {
             use_tep = TRUE;
             continue;
         }
+        if (strcmp(argv[i], "-w") == 0)
+        {
+            t30_state_to_wreck = atoi(argv[++i]);
+            continue;
+        }
     }
 
     input_wave_handle = AF_NULL_FILEHANDLE;
@@ -283,7 +316,11 @@ int main(int argc, char *argv[])
 
         i = mc->chan + 1;
         sprintf(buf, "%d%d%d%d%d%d%d%d", i, i, i, i, i, i, i, i);
-        fax_init(&mc->fax, (mc->chan & 1)  ?  FALSE  :  TRUE);
+        if (reverse_flow)
+            fax_init(&mc->fax, (mc->chan & 1)  ?  TRUE  :  FALSE);
+        else
+            fax_init(&mc->fax, (mc->chan & 1)  ?  FALSE  :  TRUE);
+        fax_set_transmit_on_idle(&mc->fax, use_transmit_on_idle);
         fax_set_tep_mode(&mc->fax, use_tep);
         t30_set_local_ident(&mc->fax.t30_state, buf);
         t30_set_header_info(&mc->fax.t30_state, page_header_info);
@@ -293,6 +330,7 @@ int main(int argc, char *argv[])
                                                         | T30_SUPPORT_215MM_WIDTH | T30_SUPPORT_255MM_WIDTH | T30_SUPPORT_303MM_WIDTH);
         t30_set_supported_resolutions(&mc->fax.t30_state, T30_SUPPORT_STANDARD_RESOLUTION | T30_SUPPORT_FINE_RESOLUTION | T30_SUPPORT_SUPERFINE_RESOLUTION
                                                         | T30_SUPPORT_R8_RESOLUTION | T30_SUPPORT_R16_RESOLUTION);
+        //t30_set_supported_modems(&mc->fax.t30_state, T30_SUPPORT_V27TER);
         if (use_ecm)
             t30_set_supported_compressions(&mc->fax.t30_state, T30_SUPPORT_T4_1D_COMPRESSION | T30_SUPPORT_T4_2D_COMPRESSION | T30_SUPPORT_T6_COMPRESSION);
         if ((mc->chan & 1))
@@ -331,8 +369,10 @@ int main(int argc, char *argv[])
         span_log_set_level(&mc->fax.logging, SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_SHOW_TAG | SPAN_LOG_SHOW_SAMPLE_TIME | SPAN_LOG_FLOW);
         span_log_set_tag(&mc->fax.logging, mc->tag);
         memset(mc->amp, 0, sizeof(mc->amp));
+        mc->total_audio_time = 0;
         mc->done = FALSE;
     }
+    time(&start_time);
     for (;;)
     {
         alldone = TRUE;
@@ -350,15 +390,18 @@ int main(int argc, char *argv[])
             {
                 mc->len = fax_tx(&mc->fax, mc->amp, SAMPLES_PER_CHUNK);
             }
-            /* The receive side always expects a full block of samples, but the
-               transmit side may not be sending any when it doesn't need to. We
-               may need to pad with some silence. */
-            if (mc->len < SAMPLES_PER_CHUNK)
+            mc->total_audio_time += SAMPLES_PER_CHUNK;
+            if (!use_transmit_on_idle)
             {
-                memset(mc->amp + mc->len, 0, sizeof(int16_t)*(SAMPLES_PER_CHUNK - mc->len));
-                mc->len = SAMPLES_PER_CHUNK;
+                /* The receive side always expects a full block of samples, but the
+                   transmit side may not be sending any when it doesn't need to. We
+                   may need to pad with some silence. */
+                if (mc->len < SAMPLES_PER_CHUNK)
+                {
+                    memset(mc->amp + mc->len, 0, sizeof(int16_t)*(SAMPLES_PER_CHUNK - mc->len));
+                    mc->len = SAMPLES_PER_CHUNK;
+                }
             }
-
             span_log_bump_samples(&mc->fax.t30_state.logging, mc->len);
             span_log_bump_samples(&mc->fax.v29rx.logging, mc->len);
             span_log_bump_samples(&mc->fax.logging, mc->len);
@@ -372,7 +415,7 @@ int main(int argc, char *argv[])
                 memset(machines[j ^ 1].amp + machines[j ^ 1].len, 0, sizeof(int16_t)*(SAMPLES_PER_CHUNK - machines[j ^ 1].len));
             if (use_line_hits)
             {
-                /* TODO: This applied very crude line hits. improve it */
+                /* TODO: This applies very crude line hits. improve it */
                 if (mc->fax.t30_state.state == 22)
                 {
                     if (++mc->error_delay == 100)
@@ -384,6 +427,8 @@ int main(int argc, char *argv[])
                     }
                 }    
             }
+            if (mc->fax.t30_state.state == t30_state_to_wreck)
+                memset(machines[j ^ 1].amp, 0, sizeof(int16_t)*SAMPLES_PER_CHUNK);
             if (fax_rx(&mc->fax, machines[j ^ 1].amp, SAMPLES_PER_CHUNK))
                 break;
             if (!mc->done)
@@ -400,6 +445,7 @@ int main(int argc, char *argv[])
         if (alldone  ||  j < FAX_MACHINES)
             break;
     }
+    time(&end_time);
     for (j = 0;  j < FAX_MACHINES;  j++)
     {
         mc = &machines[j];
@@ -423,6 +469,7 @@ int main(int argc, char *argv[])
         }
         afFreeFileSetup(filesetup);
     }
+    printf("Total audio time = %ds (wall time %ds)\n", machines[0].total_audio_time/8000, (int) (end_time - start_time));
     return  0;
 }
 /*- End of function --------------------------------------------------------*/
