@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: fsk_tests.c,v 1.29 2006/10/24 13:45:29 steveu Exp $
+ * $Id: fsk_tests.c,v 1.33 2007/04/10 16:12:20 steveu Exp $
  */
 
 /*! \page fsk_tests_page FSK modem tests
@@ -50,14 +50,18 @@ These tests allow either:
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <assert.h>
 #include <audiofile.h>
 #include <tiffio.h>
 
 #include "spandsp.h"
-#include "test_utils.h"
-#include "line_model.h"
+#include "spandsp-sim.h"
 
 #define BLOCK_LEN           160
 
@@ -172,8 +176,10 @@ int main(int argc, char *argv[])
     int modem_under_test_2;
     int log_audio;
     int channel_codec;
+    int rbs_pattern;
 
     channel_codec = MUNGE_CODEC_NONE;
+    rbs_pattern = 0;
     line_model_no = 0;
     decode_test_file = NULL;
     noise_sweep = FALSE;
@@ -193,12 +199,27 @@ int main(int argc, char *argv[])
             decode_test_file = argv[i];
             continue;
         }
+        if (strcmp(argv[i], "-l") == 0)
+        {
+            log_audio = TRUE;
+            continue;
+        }
+        if (strcmp(argv[i], "-n") == 0)
+        {
+            noise_sweep = TRUE;
+            continue;
+        }
         if (strcmp(argv[i], "-m") == 0)
         {
             i++;
             line_model_no = atoi(argv[i]);
             continue;
         }
+        if (strcmp(argv[i], "-r") == 0)
+        {
+            rbs_pattern = atoi(argv[++i]);
+            continue;
+        }
         if (strcmp(argv[i], "-s") == 0)
         {
             i++;
@@ -207,16 +228,6 @@ int main(int argc, char *argv[])
             modem_under_test_2 = atoi(argv[i]);
             continue;
         }
-        if (strcmp(argv[i], "-n") == 0)
-        {
-            noise_sweep = TRUE;
-            continue;
-        }
-        if (strcmp(argv[i], "-l") == 0)
-        {
-            log_audio = TRUE;
-            continue;
-        }
     }
 
     filesetup = AF_NULL_FILESETUP;
@@ -269,7 +280,7 @@ int main(int argc, char *argv[])
         bert_set_report(&caller_bert, 100000, reporter, (void *) (intptr_t) 1);
         bert_init(&answerer_bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
         bert_set_report(&answerer_bert, 100000, reporter, (void *) (intptr_t) 2);
-        if ((model = both_ways_line_model_init(line_model_no, (float) noise_level, line_model_no, noise_level, channel_codec)) == NULL)
+        if ((model = both_ways_line_model_init(line_model_no, (float) noise_level, line_model_no, noise_level, channel_codec, rbs_pattern)) == NULL)
         {
             fprintf(stderr, "    Failed to create line model\n");
             exit(2);
@@ -322,7 +333,7 @@ int main(int argc, char *argv[])
                                  answerer_amp,
                                  samples);
         }
-        //printf("Powers %10.5fdBm0 %10.5fdBm0\n", power_meter_dbm0(&caller_meter), power_meter_dbm0(&answerer_meter));
+        //printf("Powers %10.5fdBm0 %10.5fdBm0\n", power_meter_current_dbm0(&caller_meter), power_meter_current_dbm0(&answerer_meter));
 
         fsk_rx(&answerer_rx, caller_model_amp, samples);
         for (i = 0;  i < samples;  i++)
@@ -404,7 +415,7 @@ int main(int argc, char *argv[])
                 fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_2], TRUE, 0, (put_bit_func_t) bert_put_bit, &caller_bert);
             }
             noise_level++;
-            if ((model = both_ways_line_model_init(line_model_no, (float) noise_level, line_model_no, noise_level, channel_codec)) == NULL)
+            if ((model = both_ways_line_model_init(line_model_no, (float) noise_level, line_model_no, noise_level, channel_codec, 0)) == NULL)
             {
                 fprintf(stderr, "    Failed to create line model\n");
                 exit(2);
