@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: gsm0610_tests.c,v 1.5 2006/10/24 13:45:29 steveu Exp $
+ * $Id: gsm0610_tests.c,v 1.6 2006/11/19 14:07:27 steveu Exp $
  */
 
 /*! \file */
@@ -125,7 +125,12 @@ will be compressed to GSM 06.10 data, decompressed, and the resulting audio stor
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <assert.h>
 #include <fcntl.h>
 #include <ctype.h>
