@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: r2_mf_rx_tests.c,v 1.3 2006/10/24 13:22:03 steveu Exp $
+ * $Id: r2_mf_rx_tests.c,v 1.4 2006/11/19 14:07:27 steveu Exp $
  */
 
 /*! \file */
@@ -44,9 +44,14 @@ a fair test of performance in a real PSTN channel.
 #endif
 
 #include <stdlib.h>
-#include <math.h>
 #include <inttypes.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 #include <stdio.h>
 #include <time.h>
 #include <fcntl.h>
