@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 #
 # spandsp regression tests
 #
@@ -15,15 +15,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
-# $Id: regression_tests.sh,v 1.34 2006/10/24 13:49:53 steveu Exp $
+# $Id: regression_tests.sh,v 1.41 2007/04/30 13:13:36 steveu Exp $
 #
 
 ITUTESTS_TIF=../itutests/fax/itutests.tif
+STDOUT_DEST=xyzzy
+STDERR_DEST=xyzzy2
 
-echo Performing basic regression tests
+echo Performing basic spandsp regression tests
 echo
 
-./adsi_tests >xyzzy 2>xyzzy2
+./adsi_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -32,7 +34,7 @@ then
 fi
 echo adsi_tests completed OK
 
-./async_tests >xyzzy 2>xyzzy2
+./async_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -41,7 +43,7 @@ then
 fi
 echo async_tests completed OK
 
-./at_interpreter_tests >xyzzy 2>xyzzy2
+./at_interpreter_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -50,7 +52,7 @@ then
 fi
 echo at_interpreter_tests completed OK
 
-./awgn_tests >xyzzy 2>xyzzy2
+./awgn_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -59,7 +61,7 @@ then
 fi
 echo awgn_tests completed OK
 
-./bell_mf_rx_tests >xyzzy 2>xyzzy2
+./bell_mf_rx_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -68,7 +70,7 @@ then
 fi
 echo bell_mf_rx_tests completed OK
 
-./bell_mf_tx_tests >xyzzy 2>xyzzy2
+./bell_mf_tx_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -77,7 +79,7 @@ then
 fi
 echo bell_mf_tx_tests completed OK
 
-./bert_tests >xyzzy 2>xyzzy2
+./bert_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -86,7 +88,7 @@ then
 fi
 echo bert_tests completed OK
 
-./bit_operations_tests >xyzzy 2>xyzzy2
+./bit_operations_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -95,7 +97,7 @@ then
 fi
 echo bit_operations_tests completed OK
 
-./dc_restore_tests >xyzzy 2>xyzzy2
+./dc_restore_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -104,7 +106,7 @@ then
 fi
 echo dc_restore_tests completed OK
 
-./dds_tests >xyzzy 2>xyzzy2
+./dds_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -113,7 +115,7 @@ then
 fi
 echo dds_tests completed OK
 
-./dtmf_rx_tests >xyzzy 2>xyzzy2
+./dtmf_rx_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -122,7 +124,7 @@ then
 fi
 echo dtmf_rx_tests completed OK
 
-./dtmf_tx_tests >xyzzy 2>xyzzy2
+./dtmf_tx_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -131,7 +133,7 @@ then
 fi
 echo dtmf_tx_tests completed OK
 
-#./echo_tests >xyzzy 2>xyzzy2
+#./echo_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
 #if [ $RETVAL != 0 ]
 #then
@@ -142,7 +144,7 @@ echo dtmf_tx_tests completed OK
 echo echo_tests not enabled
 
 rm -f fax_tests_1.tif
-./fax_tests >xyzzy 2>xyzzy2
+./fax_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -159,7 +161,7 @@ then
     exit $RETVAL
 fi
 rm -f fax_tests_1.tif
-./fax_tests -e >xyzzy 2>xyzzy2
+./fax_tests -e >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -177,7 +179,7 @@ then
 fi
 echo fax_tests completed OK
 
-./fsk_tests >xyzzy 2>xyzzy2
+./fsk_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -186,7 +188,7 @@ then
 fi
 echo fsk_tests completed OK
 
-./g711_tests >xyzzy 2>xyzzy2
+./g711_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -195,7 +197,7 @@ then
 fi
 echo g711_tests completed OK
 
-./g722_tests >xyzzy 2>xyzzy2
+./g722_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -204,7 +206,7 @@ then
 fi
 echo g722_tests completed OK
 
-./g726_tests >xyzzy 2>xyzzy2
+./g726_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -213,7 +215,7 @@ then
 fi
 echo g726_tests completed OK
 
-./gsm0610_tests >xyzzy 2>xyzzy2
+./gsm0610_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -222,7 +224,7 @@ then
 fi
 echo gsm0610_tests completed OK
 
-./hdlc_tests >xyzzy 2>xyzzy2
+./hdlc_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -231,7 +233,7 @@ then
 fi
 echo hdlc_tests completed OK
 
-./ima_adpcm_tests >xyzzy 2>xyzzy2
+./ima_adpcm_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -240,7 +242,7 @@ then
 fi
 echo ima_adpcm_tests completed OK
 
-./logging_tests >xyzzy 2>xyzzy2
+./logging_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -249,7 +251,7 @@ then
 fi
 echo logging_tests completed OK
 
-./lpc10_tests >xyzzy 2>xyzzy2
+./lpc10_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -258,7 +260,7 @@ then
 fi
 echo lpc10_tests completed OK
 
-./modem_echo_tests >xyzzy 2>xyzzy2
+./modem_echo_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -267,7 +269,7 @@ then
 fi
 echo modem_echo_tests completed OK
 
-./modem_connect_tones_tests >xyzzy 2>xyzzy2
+./modem_connect_tones_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -276,7 +278,7 @@ then
 fi
 echo modem_connect_tones_tests completed OK
 
-./noise_tests >xyzzy 2>xyzzy2
+./noise_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -285,7 +287,7 @@ then
 fi
 echo noise_tests completed OK
 
-./oki_adpcm_tests >xyzzy 2>xyzzy2
+./oki_adpcm_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -294,7 +296,7 @@ then
 fi
 echo oki_adpcm_tests completed OK
 
-#./playout_tests >xyzzy 2>xyzzy2
+#./playout_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
 #if [ $RETVAL != 0 ]
 #then
@@ -304,7 +306,7 @@ echo oki_adpcm_tests completed OK
 #echo playout_tests completed OK
 echo playout_tests not enabled
 
-#./plc_tests >xyzzy 2>xyzzy2
+#./plc_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
 #if [ $RETVAL != 0 ]
 #then
@@ -314,7 +316,7 @@ echo playout_tests not enabled
 #echo plc_tests completed OK
 echo plc_tests not enabled
 
-./power_meter_tests >xyzzy 2>xyzzy2
+./power_meter_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -323,7 +325,16 @@ then
 fi
 echo power_meter_tests completed OK
 
-./r2_mf_rx_tests >xyzzy 2>xyzzy2
+./queue_tests >$STDOUT_DEST 2>$STDERR_DEST
+RETVAL=$?
+if [ $RETVAL != 0 ]
+then
+    echo queue_tests failed!
+    exit $RETVAL
+fi
+echo queue_tests completed OK
+
+./r2_mf_rx_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -332,7 +343,7 @@ then
 fi
 echo r2_mf_rx_tests completed OK
 
-./r2_mf_tx_tests >xyzzy 2>xyzzy2
+./r2_mf_tx_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -341,7 +352,7 @@ then
 fi
 echo r2_mf_tx_tests completed OK
 
-./schedule_tests >xyzzy 2>xyzzy2
+./schedule_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -350,7 +361,7 @@ then
 fi
 echo schedule_tests completed OK
 
-#./sig_tone_tests >xyzzy 2>xyzzy2
+#./sig_tone_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
 #if [ $RETVAL != 0 ]
 #then
@@ -360,7 +371,7 @@ echo schedule_tests completed OK
 #echo sig_tone_tests completed OK
 echo sig_tone_tests not enabled
 
-#./super_tone_rx_tests >xyzzy 2>xyzzy2
+#./super_tone_rx_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
 #if [ $RETVAL != 0 ]
 #then
@@ -370,7 +381,7 @@ echo sig_tone_tests not enabled
 #echo super_tone_rx_tests completed OK
 echo super_tone_rx_tests not enabled
 
-#./super_tone_tx_tests >xyzzy 2>xyzzy2
+#./super_tone_tx_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
 #if [ $RETVAL != 0 ]
 #then
@@ -380,14 +391,14 @@ echo super_tone_rx_tests not enabled
 #echo super_tone_tx_tests completed OK
 echo super_tone_tx_tests not enabled
 
-./t31_tests -r >xyzzy 2>xyzzy2
+./t31_tests -r >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
     echo t31_tests -r failed!
     exit $RETVAL
 fi
-./t31_tests -s >xyzzy 2>xyzzy2
+./t31_tests -s >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -396,8 +407,17 @@ then
 fi
 echo t31_tests completed OK
 
+./t38_core_tests >$STDOUT_DEST 2>$STDERR_DEST
+RETVAL=$?
+if [ $RETVAL != 0 ]
+then
+    echo t38_core_tests failed!
+    exit $RETVAL
+fi
+echo t38_core_tests completed OK
+
 rm -f t38.tif
-./t38_gateway_tests >xyzzy 2>xyzzy2
+./t38_gateway_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -414,7 +434,7 @@ then
     exit $RETVAL
 fi
 rm -f t38.tif
-./t38_gateway_tests -e >xyzzy 2>xyzzy2
+./t38_gateway_tests -e >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -433,7 +453,7 @@ fi
 echo t38_gateway_tests completed OK
 
 rm -f t38.tif
-./t38_gateway_to_terminal_tests >xyzzy 2>xyzzy2
+./t38_gateway_to_terminal_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -450,7 +470,7 @@ then
     exit $RETVAL
 fi
 rm -f t38.tif
-./t38_gateway_to_terminal_tests -e >xyzzy 2>xyzzy2
+./t38_gateway_to_terminal_tests -e >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -469,7 +489,7 @@ fi
 echo t38_gateway_to_terminal_tests completed OK
 
 rm -f t38.tif
-./t38_terminal_to_gateway_tests >xyzzy 2>xyzzy2
+./t38_terminal_to_gateway_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -486,7 +506,7 @@ then
     exit $RETVAL
 fi
 rm -f t38.tif
-./t38_terminal_to_gateway_tests -e >xyzzy 2>xyzzy2
+./t38_terminal_to_gateway_tests -e >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -505,7 +525,7 @@ fi
 echo t38_terminal_to_gateway_tests completed OK
 
 rm -f t38.tif
-./t38_terminal_tests >xyzzy 2>xyzzy2
+./t38_terminal_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -522,7 +542,7 @@ then
     exit $RETVAL
 fi
 rm -f t38.tif
-./t38_terminal_tests -e >xyzzy 2>xyzzy2
+./t38_terminal_tests -e >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -541,7 +561,7 @@ fi
 echo t38_terminal_tests completed OK
 
 rm -f t4_tests_receive.tif
-./t4_tests >xyzzy 2>xyzzy2
+./t4_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -559,7 +579,7 @@ then
 fi
 echo t4_tests completed OK
 
-#./time_scale_tests >xyzzy 2>xyzzy2
+#./time_scale_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
 #if [ $RETVAL != 0 ]
 #then
@@ -569,7 +589,7 @@ echo t4_tests completed OK
 #echo time_scale_tests completed OK
 echo time_scale_tests not enabled
 
-#./tone_generate_tests >xyzzy 2>xyzzy2
+#./tone_generate_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
 #if [ $RETVAL != 0 ]
 #then
@@ -579,7 +599,7 @@ echo time_scale_tests not enabled
 #echo tone_generate_tests completed OK
 echo tone_generate_tests not enabled
 
-#./v17_tests >xyzzy 2>xyzzy2
+#./v17_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
 #if [ $RETVAL != 0 ]
 #then
@@ -589,7 +609,7 @@ echo tone_generate_tests not enabled
 #echo v17_tests completed OK
 echo v17_tests not enabled
 
-#./v22bis_tests >xyzzy 2>xyzzy2
+#./v22bis_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
 #if [ $RETVAL != 0 ]
 #then
@@ -599,14 +619,14 @@ echo v17_tests not enabled
 #echo v22bis_tests completed OK
 echo v22bis_tests not enabled
 
-./v27ter_tests -s -42 2400 >xyzzy 2>xyzzy2
+./v27ter_tests -s -42 2400 >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
     echo v27ter_tests failed!
     exit $RETVAL
 fi
-./v27ter_tests -s -42 4800 >xyzzy 2>xyzzy2
+./v27ter_tests -s -42 4800 >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -615,21 +635,21 @@ then
 fi
 echo v27ter_tests completed OK
 
-./v29_tests -s -42 4800 >xyzzy 2>xyzzy2
+./v29_tests -s -42 4800 >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
     echo v29_tests failed!
     exit $RETVAL
 fi
-./v29_tests -s -42 7200 >xyzzy 2>xyzzy2
+./v29_tests -s -42 7200 >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
     echo v29_tests failed!
     exit $RETVAL
 fi
-./v29_tests -s -42 9600 >xyzzy 2>xyzzy2
+./v29_tests -s -42 9600 >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -638,7 +658,7 @@ then
 fi
 echo v29_tests completed OK
 
-#./v42_tests >xyzzy 2>xyzzy2
+#./v42_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
 #if [ $RETVAL != 0 ]
 #then
@@ -657,7 +677,7 @@ then
 fi
 echo v42bis_tests completed OK
 
-./v8_tests >xyzzy 2>xyzzy2
+./v8_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -666,7 +686,7 @@ then
 fi
 echo v8_tests completed OK
 
-./vector_float_tests >xyzzy 2>xyzzy2
+./vector_float_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
@@ -675,7 +695,7 @@ then
 fi
 echo vector_float_tests completed OK
 
-./vector_int_tests >xyzzy 2>xyzzy2
+./vector_int_tests >$STDOUT_DEST 2>$STDERR_DEST
 RETVAL=$?
 if [ $RETVAL != 0 ]
 then
