@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: schedule_tests.c,v 1.12 2006/10/24 13:45:29 steveu Exp $
+ * $Id: schedule_tests.c,v 1.13 2006/11/19 14:07:27 steveu Exp $
  */
 
 /*! \page schedule_tests_page Event scheduler tests
@@ -41,7 +41,12 @@
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 #include <tiffio.h>
 
 #include "spandsp.h"
