@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: time_scale_tests.c,v 1.13 2006/10/24 13:22:03 steveu Exp $
+ * $Id: time_scale_tests.c,v 1.14 2006/11/19 14:07:27 steveu Exp $
  */
 
 /*! \page time_scale_tests_page Time scaling tests
@@ -44,8 +44,12 @@ This file also contains 8000 sample/second 16 bits/sample linear audio.
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <audiofile.h>
 #include <tiffio.h>
 
