@@ -1,4 +1,5 @@
 #define ENABLE_V17
+//#define ADD_MAINS_INTERFERENCE
 /*
  * SpanDSP - a series of DSP components for telephony
  *
@@ -23,7 +24,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v17_tests.c,v 1.59 2006/10/24 13:45:29 steveu Exp $
+ * $Id: v17_tests.c,v 1.66 2007/04/03 12:59:33 steveu Exp $
  */
 
 /*! \page v17_tests_page V.17 modem tests
@@ -59,14 +60,19 @@ display of modem status is maintained.
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <assert.h>
 #include <audiofile.h>
 #include <tiffio.h>
 
 #include "spandsp.h"
-#include "test_utils.h"
-#include "line_model.h"
+#include "spandsp-sim.h"
+
 #if defined(ENABLE_GUI)
 #include "modem_monitor.h"
 #include "line_model_monitor.h"
@@ -173,33 +179,16 @@ static void v17putbit(void *user_data, int bit)
         return;
     }
 
-#if 0
-    printf("Rx bit %d - %d - %d\n", rx_bits++, bit, tx_bits);
-#else
     if (decode_test_file)
         printf("Rx bit %d - %d\n", rx_bits++, bit);
     else
         bert_put_bit(&bert, bit);
-#endif
 }
 /*- End of function --------------------------------------------------------*/
 
 static int v17getbit(void *user_data)
 {
-#if 0
-    if (++tx_bits >= 50000)
-    {
-        tx_bits = 0;
-        return PUTBIT_END_OF_DATA;
-    }
-//    if (tx_bits < 200)
-//        return (tx_bits & 1);
-    if (tx_bits == 1)
-        return 0;
-    return 1;
-#else
     return bert_get_bit(&bert);
-#endif
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -277,8 +266,11 @@ int main(int argc, char *argv[])
     int line_model_no;
     int log_audio;
     int channel_codec;
+    int rbs_pattern;
+    float x;
 
     channel_codec = MUNGE_CODEC_NONE;
+    rbs_pattern = 0;
     test_bps = 14400;
     tep = FALSE;
     line_model_no = 0;
@@ -329,6 +321,11 @@ int main(int argc, char *argv[])
             noise_level = atoi(argv[++i]);
             continue;
         }
+        if (strcmp(argv[i], "-r") == 0)
+        {
+            rbs_pattern = atoi(argv[++i]);
+            continue;
+        }
         if (strcmp(argv[i], "-s") == 0)
         {
             signal_level = atoi(argv[++i]);
@@ -378,6 +375,21 @@ int main(int argc, char *argv[])
             fprintf(stderr, "    Cannot open wave file '%s'\n", decode_test_file);
             exit(2);
         }
+        if ((x = afGetFrameSize(inhandle, AF_DEFAULT_TRACK, 1)) != 2.0)
+        {
+            printf("    Unexpected frame size in speech file '%s'\n", decode_test_file);
+            exit(2);
+        }
+        if ((x = afGetRate(inhandle, AF_DEFAULT_TRACK)) != (float) SAMPLE_RATE)
+        {
+            printf("    Unexpected sample rate in speech file '%s'\n", decode_test_file);
+            exit(2);
+        }
+        if ((x = afGetChannels(inhandle, AF_DEFAULT_TRACK)) != 1.0)
+        {
+            printf("    Unexpected number of channels in speech file '%s'\n", decode_test_file);
+            exit(2);
+        }
     }
     else
     {
@@ -391,11 +403,15 @@ int main(int argc, char *argv[])
         bert_init(&bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
         bert_set_report(&bert, 10000, reporter, NULL);
 
-        if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level, channel_codec)) == NULL)
+        if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level, channel_codec, rbs_pattern)) == NULL)
         {
             fprintf(stderr, "    Failed to create line model\n");
             exit(2);
         }
+        one_way_line_model_set_dc(line_model, 0.0f);
+#if defined(ADD_MAINS_INTERFERENCE)
+        one_way_line_model_set_mains_pickup(line_model, 50, -40.0f);
+#endif
     }
 
     v17_rx_init(&rx, test_bps, v17putbit, &rx);
@@ -469,15 +485,16 @@ int main(int argc, char *argv[])
                 v17_tx_restart(&tx, test_bps, tep, TRUE);
                 v17_tx_power(&tx, signal_level);
                 v17_rx_restart(&rx, test_bps, TRUE);
-                rx.eq_put_step = rand()%(192*10/3);
+                //rx.eq_put_step = rand()%(192*10/3);
                 bert_init(&bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
                 bert_set_report(&bert, 10000, reporter, NULL);
             }
+            one_way_line_model(line_model, amp, gen_amp, samples);
             if (log_audio)
             {
                 outframes = afWriteFrames(outhandle,
                                           AF_DEFAULT_TRACK,
-                                          gen_amp,
+                                          amp,
                                           samples);
                 if (outframes != samples)
                 {
@@ -485,7 +502,6 @@ int main(int argc, char *argv[])
                     exit(2);
                 }
             }
-            one_way_line_model(line_model, amp, gen_amp, samples);
         }
 #if defined(ENABLE_GUI)
         if (use_gui  &&  !decode_test_file)
