@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v22bis_tests.c,v 1.36 2006/10/24 13:45:29 steveu Exp $
+ * $Id: v22bis_tests.c,v 1.39 2007/04/03 12:59:33 steveu Exp $
  */
 
 /*! \page v22bis_tests_page V.22bis modem tests
@@ -50,14 +50,19 @@ display of modem status is maintained.
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
+#include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
 #include <math.h>
+#endif
 #include <assert.h>
 #include <audiofile.h>
 #include <tiffio.h>
 
 #include "spandsp.h"
-#include "test_utils.h"
-#include "line_model.h"
+#include "spandsp-sim.h"
+
 #if defined(ENABLE_GUI)
 #include "modem_monitor.h"
 #endif
@@ -336,7 +341,7 @@ int main(int argc, char *argv[])
     }
 #endif
 
-    if ((model = both_ways_line_model_init(line_model_no, (float) noise_level, line_model_no, (float) noise_level, channel_codec)) == NULL)
+    if ((model = both_ways_line_model_init(line_model_no, (float) noise_level, line_model_no, (float) noise_level, channel_codec, 0)) == NULL)
     {
         fprintf(stderr, "    Failed to create line model\n");
         exit(2);
