@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v27ter_tests.c,v 1.69 2006/10/24 13:22:04 steveu Exp $
+ * $Id: v27ter_tests.c,v 1.73 2007/04/03 12:59:33 steveu Exp $
  */
 
 /*! \page v27ter_tests_page V.27ter modem tests
@@ -58,14 +58,19 @@ display of modem status is maintained.
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 #include <assert.h>
 #include <audiofile.h>
 #include <tiffio.h>
 
 #include "spandsp.h"
-#include "test_utils.h"
-#include "line_model.h"
+#include "spandsp-sim.h"
+
 #if defined(ENABLE_GUI)
 #include "modem_monitor.h"
 #include "line_model_monitor.h"
@@ -265,8 +270,11 @@ int main(int argc, char *argv[])
     int block;
     int log_audio;
     int channel_codec;
+    int rbs_pattern;
+    float x;
 
     channel_codec = MUNGE_CODEC_NONE;
+    rbs_pattern = 0;
     test_bps = 4800;
     tep = FALSE;
     line_model_no = 0;
@@ -293,11 +301,6 @@ int main(int argc, char *argv[])
             decode_test_file = argv[++i];
             continue;
         }
-        if (strcmp(argv[i], "-t") == 0)
-        {
-            tep = TRUE;
-            continue;
-        }
         if (strcmp(argv[i], "-g") == 0)
         {
             use_gui = TRUE;
@@ -323,6 +326,16 @@ int main(int argc, char *argv[])
             signal_level = atoi(argv[++i]);
             continue;
         }
+        if (strcmp(argv[i], "-r") == 0)
+        {
+            rbs_pattern = atoi(argv[++i]);
+            continue;
+        }
+        if (strcmp(argv[i], "-t") == 0)
+        {
+            tep = TRUE;
+            continue;
+        }
         if (strcmp(argv[i], "4800") == 0)
             test_bps = 4800;
         else if (strcmp(argv[i], "2400") == 0)
@@ -363,6 +376,21 @@ int main(int argc, char *argv[])
             fprintf(stderr, "    Cannot open wave file '%s'\n", decode_test_file);
             exit(2);
         }
+        if ((x = afGetFrameSize(inhandle, AF_DEFAULT_TRACK, 1)) != 2.0)
+        {
+            printf("    Unexpected frame size in speech file '%s'\n", decode_test_file);
+            exit(2);
+        }
+        if ((x = afGetRate(inhandle, AF_DEFAULT_TRACK)) != (float) SAMPLE_RATE)
+        {
+            printf("    Unexpected sample rate in speech file '%s'\n", decode_test_file);
+            exit(2);
+        }
+        if ((x = afGetChannels(inhandle, AF_DEFAULT_TRACK)) != 1.0)
+        {
+            printf("    Unexpected number of channels in speech file '%s'\n", decode_test_file);
+            exit(2);
+        }
     }
     else
     {
@@ -375,7 +403,7 @@ int main(int argc, char *argv[])
         bert_init(&bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
         bert_set_report(&bert, 10000, reporter, NULL);
 
-        if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level, channel_codec)) == NULL)
+        if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level, channel_codec, rbs_pattern)) == NULL)
         {
             fprintf(stderr, "    Failed to create line model\n");
             exit(2);
@@ -457,7 +485,7 @@ int main(int argc, char *argv[])
                 bert_init(&bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
                 bert_set_report(&bert, 10000, reporter, NULL);
                 one_way_line_model_release(line_model);
-                if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level, channel_codec)) == NULL)
+                if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level, channel_codec, 0)) == NULL)
                 {
                     fprintf(stderr, "    Failed to create line model\n");
                     exit(2);
